#!/usr/bin/python
# coding:utf-8
from hbshare.quant.CChen.load_data import load_funds_data, load_funds_alpha
from hbshare.quant.CChen.fund_perf import ret
import pandas as pd
import numpy as np
import pyecharts.options as opts
from datetime import datetime, timedelta
from pyecharts.charts import Line, Grid, Bar, Boxplot, Pie
from pyecharts.globals import ThemeType
from pyecharts.commons.utils import JsCode
import pymysql
from sqlalchemy import create_engine
from hbshare.quant.CChen.cons import index_underlying, underlying_code
from hbshare.quant.CChen.fut import (
    wind_fin_fut_index,
    wind_com_fut_sec_index,
    fin_fut_daily_k_by_contracts,
    fut_com_amount_by_ex,
    rolling_volatility,
)
from hbshare.quant.CChen.load_hsjy import load_fut_fin
from hbshare.quant.CChen.stk import load_index
from hbshare.quant.CChen.cons import sql_write_path_hb, HSJY_VIRTUAL_CONTRACT

pymysql.install_as_MySQLdb()


def nav_lines(
        fund_list, start_date, end_date, title='', zz=False, axis_cross=None,
        db_path=sql_write_path_hb['daily'], cal_path=sql_write_path_hb['work'], all_selected=True, print_info=True
):
    if axis_cross:
        axis_cross = opts.TooltipOpts(trigger="axis", axis_pointer_type="cross")

    if zz:
        funds_data = load_funds_alpha(
            fund_list=fund_list,
            first_date=start_date,
            end_date=end_date,
            db_path=db_path,
            cal_db_path=cal_path,
            print_info=print_info,
        )['eav']
    else:
        funds_data = load_funds_data(
            fund_list=fund_list,
            first_date=start_date,
            end_date=end_date,
            db_path=db_path,
            cal_db_path=cal_path,
            print_info=print_info,
            # freq='',
            # fillna=False
        )

    web = Line(
        init_opts=opts.InitOpts(
            page_title=title,
            width='700px',
            height='500px',
            theme=ThemeType.CHALK
        )
    ).set_global_opts(
        tooltip_opts=opts.TooltipOpts(is_show=True),
        toolbox_opts=opts.ToolboxOpts(is_show=True),
        xaxis_opts=opts.AxisOpts(type_="category"),
        yaxis_opts=opts.AxisOpts(
            type_="value",
            axistick_opts=opts.AxisTickOpts(is_show=True),
            splitline_opts=opts.SplitLineOpts(is_show=True),
        ),
    ).add_xaxis(
        xaxis_data=funds_data['t_date'].tolist()
    )
    funds = funds_data.columns.tolist()

    funds.remove('t_date')
    for j in funds:
        nav_data = funds_data[j] / funds_data[funds_data[j] > 0][j].tolist()[0]
        web.add_yaxis(
            series_name=j,
            y_axis=nav_data.round(4).tolist(),
            symbol="emptyCircle",
            is_symbol_show=True,
            label_opts=opts.LabelOpts(is_show=False),
            is_selected=all_selected
        )

    web.set_global_opts(
        xaxis_opts=opts.AxisOpts(
            is_scale=True,
            type_="category", boundary_gap=False
        ),
        yaxis_opts=opts.AxisOpts(
            is_scale=True,
            splitarea_opts=opts.SplitAreaOpts(
                is_show=True, areastyle_opts=opts.AreaStyleOpts(opacity=1)
            ),
        ),
        legend_opts=opts.LegendOpts(
            # type_='scroll',
            pos_top='5%'
        ),
        datazoom_opts=[
            opts.DataZoomOpts(range_start=0, range_end=100),
            # opts.DataZoomOpts(pos_left="5%", xaxis_index=0),
            # opts.DataZoomOpts(pos_right="5%", xaxis_index=1),
            opts.DataZoomOpts(type_="inside")
        ],
        title_opts=opts.TitleOpts(title=title),
        tooltip_opts=axis_cross
    )

    return web


def gen_grid(
        end_date, funds, zz=False, lookback_years=3, grid_width=1200, grid_height=500, pos_top=23, axis_cross=None,
        db_path=sql_write_path_hb['work'], cal_path=sql_write_path_hb['daily'], all_selected=True, print_info=True
):
    if axis_cross:
        axis_cross = opts.TooltipOpts(trigger="axis", axis_pointer_type="cross")

    engine = create_engine(db_path)
    start_date = end_date - timedelta(days=365 * lookback_years + 7)
    start_date2 = end_date - timedelta(days=365 + 7)
    if len(funds) > 1:
        fund_list = pd.read_sql_query(
            'select * from fund_list where `name` in ' + str(tuple(funds)) + ' order by `name`', engine
        )
    else:
        fund_list = pd.read_sql_query(
            'select * from fund_list where `name`="' + str(funds[0]) + '"', engine
        )

    grid_nav = Grid(init_opts=opts.InitOpts(width=str(grid_width) + "px", height=str(grid_height) + "px"))

    web = nav_lines(
        fund_list=fund_list,
        start_date=start_date,
        end_date=end_date,
        zz=zz,
        db_path=db_path,
        cal_path=cal_path,
        all_selected=all_selected,
        print_info=print_info,
    )

    web2 = nav_lines(
        fund_list=fund_list,
        start_date=start_date2,
        end_date=end_date,
        zz=zz,
        db_path=db_path,
        cal_path=cal_path,
        all_selected=all_selected,
        print_info=print_info,
    )

    grid_nav.add(
        web.set_global_opts(
            title_opts=opts.TitleOpts(
                title='近' + str(lookback_years) + '年: '
                      + start_date.strftime('%Y/%m/%d')
                      + '~' + end_date.strftime('%Y/%m/%d'),
                pos_right="5%"
            ),
            tooltip_opts=axis_cross,
            legend_opts=opts.LegendOpts(
                pos_top="7%"
            )
        ),
        grid_opts=opts.GridOpts(pos_left='55%', pos_top=str(pos_top) + "%")
    ).add(
        web2.set_global_opts(
            title_opts=opts.TitleOpts(
                title='近一年: '
                      + start_date2.strftime('%Y/%m/%d') + '~' + end_date.strftime('%Y/%m/%d')
                ,
                pos_left="5%"
            ),
            tooltip_opts=axis_cross,
            legend_opts=opts.LegendOpts(
                pos_top="7%"
            )
        ),
        grid_opts=opts.GridOpts(pos_right='55%', pos_top=str(pos_top) + "%")
    )
    return grid_nav


def index_fut_basis_wind(
        index_code, start_date, end_date,
        sql_path=sql_write_path_hb['daily'], table='futures_wind',
        grid_width=1200, grid_height=400, zoom_left=0, zoom_right=100,
        chart_pos_left=10, chart_pos_right=15,
):
    engine = create_engine(sql_path)

    data_raw = pd.read_sql_query(
        'select * from ' + table
        + ' where `t_date`<=' + end_date.strftime('%Y%m%d')
        + ' and `t_date`>=' + start_date.strftime('%Y%m%d')
        + ' and `product`=\'' + index_code + '\' order by `t_date`',
        engine
    )

    data_00 = data_raw[data_raw['symbol'] == index_code + '00.CFE'].reset_index(drop=True)
    data_01 = data_raw[data_raw['symbol'] == index_code + '01.CFE'].reset_index(drop=True)
    data_02 = data_raw[data_raw['symbol'] == index_code + '02.CFE'].reset_index(drop=True)
    data_03 = data_raw[data_raw['symbol'] == index_code + '03.CFE'].reset_index(drop=True)

    data_00['remain'] = (data_00['delist_date'] - data_00['t_date']).map(lambda x: x.days)
    data_00['basis'] = round(-data_00['basis'] / data_00['underlying'] / data_00['remain'] * 365 * 100, 2)
    data_00['basis_chg'] = round(
        -(
            data_00['close'] / data_00['close'].shift(1)
            - data_00['underlying'] / data_00['underlying'].shift(1)
        ) * 100, 2
    )

    data_01['remain'] = (data_01['delist_date'] - data_01['t_date']).map(lambda x: x.days)
    data_01['basis'] = round(-data_01['basis'] / data_01['underlying'] / data_01['remain'] * 365 * 100, 2)
    data_01['basis_chg'] = round(
        (
                data_01['close'] / data_01['close'].shift(1)
                - data_01['underlying'] / data_01['underlying'].shift(1)
        ) * 100, 2
    )

    data_02['remain'] = (data_02['delist_date'] - data_02['t_date']).map(lambda x: x.days)
    data_02['basis'] = round(-data_02['basis'] / data_02['underlying'] / data_02['remain'] * 365 * 100, 2)
    data_02['basis_chg'] = round(
        (
                data_02['close'] / data_02['close'].shift(1)
                - data_02['underlying'] / data_02['underlying'].shift(1)
        ) * 100, 2
    )

    data_03['remain'] = (data_03['delist_date'] - data_03['t_date']).map(lambda x: x.days)
    data_03['basis'] = round(-data_03['basis'] / data_03['underlying'] / data_03['remain'] * 365 * 100, 2)
    data_03['basis_chg'] = round(
        (
                data_03['close'] / data_03['close'].shift(1)
                - data_03['underlying'] / data_03['underlying'].shift(1)
        ) * 100, 2
    )

    web = Line(
        # init_opts=opts.InitOpts(width="1200px", height="400px")
    ).add_xaxis(
        xaxis_data=data_00['t_date'].tolist()
    ).add_yaxis(
        series_name=index_underlying[index_code],
        y_axis=(data_00['underlying'] / data_00['underlying'][0]).round(4),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=0,
        is_selected=False,
    ).add_yaxis(
        series_name='当月',
        y_axis=(data_00['close'] / data_00['close'][0]).round(4),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=0,
        is_selected=False,
    ).add_yaxis(
        series_name='次月',
        y_axis=(data_01['close'] / data_01['close'][0]).round(4),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=0,
        is_selected=False,
    ).add_yaxis(
        series_name='季月',
        y_axis=(data_02['close'] / data_02['close'][0]).round(4),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=0,
        is_selected=False,
    ).add_yaxis(
        series_name='次季',
        y_axis=(data_03['close'] / data_03['close'][0]).round(4),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=0,
        is_selected=False,
    ).extend_axis(
        yaxis=opts.AxisOpts(
            name='年化基差',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            position="right",
            # interval=5,
            min_='dataMin',
            max_='dataMax',
        )
    ).extend_axis(
            yaxis=opts.AxisOpts(
                name='基差变化',
                axislabel_opts=opts.LabelOpts(formatter="{value} %"),
                position="right",
                offset=80,
                min_='dataMin',
                max_='dataMax',
                # interval=5
                )
    ).set_global_opts(
        title_opts=opts.TitleOpts(
            title=data_raw['t_date'][len(data_raw) - 1]
        ),
        tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        # toolbox_opts=opts.ToolboxOpts(is_show=True),
        xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False),
        yaxis_opts=opts.AxisOpts(
            axislabel_opts=opts.LabelOpts(formatter="{value}"),
            min_='dataMin',
            max_='dataMax',
        ),
        datazoom_opts=[
            opts.DataZoomOpts(range_start=zoom_left, range_end=zoom_right),
            # opts.DataZoomOpts(pos_left="5%", xaxis_index=0),
            # opts.DataZoomOpts(pos_right="5%", xaxis_index=1),
            opts.DataZoomOpts(type_="inside")
        ],
        legend_opts=opts.LegendOpts(
            pos_top="7%",
            pos_left='10%',
            pos_right='10%'
        )
    )

    web2 = Line().add_xaxis(
        xaxis_data=data_00['t_date'].tolist()
    ).add_yaxis(
        series_name='当月基差',
        y_axis=data_00['basis'].tolist(),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=1,
        is_selected=False,
    ).add_yaxis(
        series_name='次月基差',
        y_axis=data_01['basis'].tolist(),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=1,
        # is_selected=False,
    ).add_yaxis(
        series_name='季月基差',
        y_axis=data_02['basis'].tolist(),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=1,
        # is_selected=False,
    ).add_yaxis(
        series_name='次季基差',
        y_axis=data_03['basis'].tolist(),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=1,
        # is_selected=False,
    ).add_yaxis(
        series_name='4基差简单平均',
        y_axis=(data_00['basis'] + data_01['basis'] + data_02['basis'] + data_03['basis']).round(2).tolist(),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=1,
        # is_selected=False,
    ).add_yaxis(
        series_name='4基差成交量加权平均',
        y_axis=(
            (
                    data_00['basis'] * data_00['volume']
                    + data_01['basis'] * data_01['volume']
                    + data_02['basis'] * data_02['volume']
                    + data_03['basis'] * data_03['volume']
            ) / (data_00['volume'] + data_01['volume'] + data_02['volume'] + data_03['volume'])
        ).round(2).tolist(),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=1,
        # is_selected=False,
    )

    web3 = Bar().add_xaxis(
        xaxis_data=data_00['t_date'].tolist()
    ).add_yaxis(
        series_name='变化/当月',
        y_axis=data_00['basis_chg'].tolist(),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=2,
        is_selected=False,
    ).add_yaxis(
        series_name='变化/次月',
        y_axis=data_01['basis_chg'].tolist(),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=2
    ).add_yaxis(
        series_name='变化/季月',
        y_axis=data_02['basis_chg'].tolist(),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=2
    ).add_yaxis(
        series_name='变化/次季',
        y_axis=data_03['basis_chg'].tolist(),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=2
    )

    web.overlap(web2).overlap(web3)

    grid = (
        Grid(init_opts=opts.InitOpts(
            width=str(grid_width) + "px", height=str(grid_height) + "px")
        ).add(
            web, grid_opts=opts.GridOpts(
                pos_right=str(chart_pos_right) + "%",
                pos_left=str(chart_pos_left) + "%",
                pos_top='25%',
            ),
            is_control_axis_index=True
        )
    )
    return grid


def index_fut_basis_wind_detail(
        index_code, start_date, end_date,
        sql_path=sql_write_path_hb['daily'], table='futures_wind',
        grid_width=1200, grid_height=400, zoom_left=0, zoom_right=100,
        chart_pos_left=10, chart_pos_right=15,
):
    engine = create_engine(sql_path)

    data_raw = pd.read_sql_query(
        'select * from ' + table
        + ' where `t_date`<=' + end_date.strftime('%Y%m%d')
        + ' and `t_date`>=' + start_date.strftime('%Y%m%d')
        + ' and `product`=\'' + index_code + '\' order by `t_date`',
        engine
    )
    contracts = data_raw['code'].drop_duplicates().sort_values(ascending=False).tolist()

    data_o = data_raw[['t_date', 'underlying']].drop_duplicates(['t_date']).reset_index(drop=True)
    web = Line(
        # init_opts=opts.InitOpts(width="1200px", height="400px")
    ).add_xaxis(
        xaxis_data=data_o['t_date']
    ).add_yaxis(
        series_name=index_underlying[index_code],
        y_axis=(data_o['underlying'] / data_o['underlying'][0]).round(4),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=0,
        # is_selected=False,
    )

    data_bucket = {}
    for c in range(min(len(contracts), 4)):
        data_bucket[contracts[c]] = data_o[['t_date']].merge(
            data_raw[data_raw['code'] == contracts[c]].reset_index(drop=True),
            on='t_date', how='left'
        )

        # data_bucket[contracts[c]]['remain'] = (
        #         data_bucket[contracts[c]]['delist_date'] - data_bucket[contracts[c]]['t_date']
        # ).map(lambda x: x.days)
        data_bucket[contracts[c]]['basis'] = round(
            - data_bucket[contracts[c]]['basis']
            / data_bucket[contracts[c]]['underlying']
            # / data_bucket[contracts[c]]['remain'] * 365
            * 100,
            2
        )
        data_bucket[contracts[c]]['basis_chg'] = round(
            (
                    data_bucket[contracts[c]]['close'] / data_bucket[contracts[c]]['close'].shift(1)
                    - data_bucket[contracts[c]]['underlying'] / data_bucket[contracts[c]]['underlying'].shift(1)
            ) * 100, 2
        )
        if c == 1:
            selected = True
        else:
            selected = False
        web.add_yaxis(
            series_name=contracts[c][2:6],
            y_axis=(
                    data_bucket[contracts[c]]['close']
                    / data_bucket[contracts[c]][data_bucket[contracts[c]]['close'] > 0]['close'].tolist()[0]
            ).round(4),
            label_opts=opts.LabelOpts(is_show=False),
            yaxis_index=0,
            is_selected=selected,
        )

    web.extend_axis(
        yaxis=opts.AxisOpts(
            name='基差',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            position="right",
            # interval=5,
            min_='dataMin',
            max_='dataMax',
        )
    ).extend_axis(
        yaxis=opts.AxisOpts(
            name='基差变化',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            position="right",
            offset=80,
            min_='dataMin',
            max_='dataMax',
            # interval=5
        )
    ).set_global_opts(
        title_opts=opts.TitleOpts(
            title=data_raw['t_date'][len(data_raw) - 1]
        ),
        tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        # toolbox_opts=opts.ToolboxOpts(is_show=True),
        xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False),
        yaxis_opts=opts.AxisOpts(
            axislabel_opts=opts.LabelOpts(formatter="{value}"),
            min_='dataMin',
            max_='dataMax',
        ),
        datazoom_opts=[
            opts.DataZoomOpts(range_start=zoom_left, range_end=zoom_right),
            # opts.DataZoomOpts(pos_left="5%", xaxis_index=0),
            # opts.DataZoomOpts(pos_right="5%", xaxis_index=1),
            opts.DataZoomOpts(type_="inside")
        ],
        legend_opts=opts.LegendOpts(
            pos_top="7%",
            pos_left='10%',
            pos_right='10%'
        )
    )

    web2 = Line().add_xaxis(
        xaxis_data=data_o['t_date']
    )
    web3 = Bar().add_xaxis(
        xaxis_data=data_o['t_date']
    )
    for c in range(min(len(contracts), 4)):
        if c == 1:
            selected = True
        else:
            selected = False
        web2.add_yaxis(
            series_name=contracts[c][2:6] + '基差',
            y_axis=data_bucket[contracts[c]]['basis'].tolist(),
            label_opts=opts.LabelOpts(is_show=False),
            yaxis_index=1,
            is_selected=False,
        )

        web3.add_yaxis(
            series_name=contracts[c][2:6] + '变化',
            y_axis=data_bucket[contracts[c]]['basis_chg'].tolist(),
            label_opts=opts.LabelOpts(is_show=False),
            yaxis_index=2,
            is_selected=selected,
            # areastyle_opts=opts.AreaStyleOpts(opacity=0.5),
        )

    web.overlap(web2).overlap(web3)

    grid = (
        Grid(init_opts=opts.InitOpts(
            width=str(grid_width) + "px", height=str(grid_height) + "px")
        ).add(
            web, grid_opts=opts.GridOpts(
                pos_right=str(chart_pos_right) + "%",
                pos_left=str(chart_pos_left) + "%",
                pos_top='25%',
            ),
            is_control_axis_index=True
        )
    )
    return grid


def index_fut_basis_hsjy(
        index_code, start_date, end_date,
        grid_width=1200, grid_height=400, zoom_left=0, zoom_right=100,
        chart_pos_left=10, chart_pos_right=15,
):
    data = load_fut_fin(index_code=index_code, start_date=start_date, end_date=end_date)
    data_o = load_index(index_list=[underlying_code[index_code]], start_date=start_date, end_date=end_date)
    data_o['TDATE'] = pd.to_datetime(data_o['TDATE']).dt.date
    data['TDATE'] = pd.to_datetime(data['TDATE']).dt.date

    contracts = data[['CODE']].drop_duplicates()
    contracts['keep'] = contracts['CODE'].apply(lambda x: True if 'y' in x.lower() else False)
    contracts = contracts[contracts['keep']].sort_values(by='CODE', ascending=False)['CODE'].tolist()

    data = data.pivot(index='TDATE', columns='CODE', values='CLOSE')

    web = Line(
        # init_opts=opts.InitOpts(width="1200px", height="400px")
    ).add_xaxis(
        xaxis_data=data_o['TDATE']
    ).add_yaxis(
        series_name=index_underlying[index_code],
        y_axis=(data_o['CLOSE'] / data_o['CLOSE'][0]).round(4),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=0,
        # is_selected=False,
    )

    data_bucket = {}
    for c in range(min(len(contracts), 4)):
        data_bucket[contracts[c]] = data_o[['TDATE', 'CLOSE']].merge(
            data[contracts[c]].reset_index(),
            on='TDATE', how='left'
        )

        data_bucket[contracts[c]]['basis'] = round(
            (data_bucket[contracts[c]][contracts[c]] - data_bucket[contracts[c]]['CLOSE'])
            / data_bucket[contracts[c]]['CLOSE'] * 100,
            2
        )
        data_bucket[contracts[c]]['basis_chg'] = round(
            (
                    data_bucket[contracts[c]][contracts[c]] / data_bucket[contracts[c]][contracts[c]].shift(1)
                    - data_bucket[contracts[c]]['CLOSE'] / data_bucket[contracts[c]]['CLOSE'].shift(1)
            ) * 100, 2
        )
        if c == 1:
            selected = True
        else:
            selected = False
        web.add_yaxis(
            series_name=contracts[c][2:6],
            y_axis=(
                    data_bucket[contracts[c]][contracts[c]]
                    / data_bucket[contracts[c]][data_bucket[contracts[c]][contracts[c]] > 0][contracts[c]].tolist()[0]
            ).round(4),
            label_opts=opts.LabelOpts(is_show=False),
            yaxis_index=0,
            is_selected=selected,
        )

    web.extend_axis(
        yaxis=opts.AxisOpts(
            name='基差',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            position="right",
            # interval=5,
            min_='dataMin',
            max_='dataMax',
        )
    ).extend_axis(
        yaxis=opts.AxisOpts(
            name='基差变化',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            position="right",
            offset=80,
            min_='dataMin',
            max_='dataMax',
            # interval=5
        )
    ).set_global_opts(
        title_opts=opts.TitleOpts(
            title=data_o['TDATE'][len(data_o) - 1]
        ),
        tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        # toolbox_opts=opts.ToolboxOpts(is_show=True),
        xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False),
        yaxis_opts=opts.AxisOpts(
            axislabel_opts=opts.LabelOpts(formatter="{value}"),
            min_='dataMin',
            max_='dataMax',
        ),
        datazoom_opts=[
            opts.DataZoomOpts(range_start=zoom_left, range_end=zoom_right),
            # opts.DataZoomOpts(pos_left="5%", xaxis_index=0),
            # opts.DataZoomOpts(pos_right="5%", xaxis_index=1),
            opts.DataZoomOpts(type_="inside")
        ],
        legend_opts=opts.LegendOpts(
            pos_top="7%",
            pos_left='10%',
            pos_right='10%'
        )
    )

    web2 = Line().add_xaxis(
        xaxis_data=data_o['TDATE']
    )
    web3 = Bar().add_xaxis(
        xaxis_data=data_o['TDATE']
    )
    for c in range(min(len(contracts), 4)):
        if c == 1:
            selected = True
        else:
            selected = False
        web2.add_yaxis(
            series_name=contracts[c][2:6] + '基差',
            y_axis=data_bucket[contracts[c]]['basis'].tolist(),
            label_opts=opts.LabelOpts(is_show=False),
            yaxis_index=1,
            is_selected=False,
        )

        web3.add_yaxis(
            series_name=contracts[c][2:6] + '变化',
            y_axis=data_bucket[contracts[c]]['basis_chg'].tolist(),
            label_opts=opts.LabelOpts(is_show=False),
            yaxis_index=2,
            is_selected=selected,
            # areastyle_opts=opts.AreaStyleOpts(opacity=0.5),
        )

    web.overlap(web2).overlap(web3)

    grid = (
        Grid(init_opts=opts.InitOpts(
            width=str(grid_width) + "px", height=str(grid_height) + "px")
        ).add(
            web, grid_opts=opts.GridOpts(
                pos_right=str(chart_pos_right) + "%",
                pos_left=str(chart_pos_left) + "%",
                pos_top='25%',
            ),
            is_control_axis_index=True
        )
    )
    return grid


def index_fut_basis_hsjy_detail(
        index_code, start_date, end_date,
        grid_width=1200, grid_height=400, zoom_left=0, zoom_right=100,
        chart_pos_left=10, chart_pos_right=15,
):

    data = load_fut_fin(index_code=index_code, start_date=start_date, end_date=end_date)
    data_o = load_index(index_list=[underlying_code[index_code]], start_date=start_date, end_date=end_date)
    data_o['TDATE'] = pd.to_datetime(data_o['TDATE']).dt.date
    data['TDATE'] = pd.to_datetime(data['TDATE']).dt.date

    contracts = data[['CODE']].drop_duplicates()
    contracts['keep'] = contracts['CODE'].apply(lambda x: False if 'y' in x.lower() else True)
    contracts = contracts[contracts['keep']].sort_values(by='CODE', ascending=False)['CODE'].tolist()

    data = data.pivot(index='TDATE', columns='CODE', values='CLOSE')

    web = Line(
        # init_opts=opts.InitOpts(width="1200px", height="400px")
    ).add_xaxis(
        xaxis_data=data_o['TDATE']
    ).add_yaxis(
        series_name=index_underlying[index_code],
        y_axis=(data_o['CLOSE'] / data_o['CLOSE'][0]).round(4),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=0,
        # is_selected=False,
    )

    data_bucket = {}
    for c in range(min(len(contracts), 4)):
        data_bucket[contracts[c]] = data_o[['TDATE', 'CLOSE']].merge(
            data[contracts[c]].reset_index(),
            on='TDATE', how='left'
        )

        data_bucket[contracts[c]]['basis'] = round(
            (data_bucket[contracts[c]][contracts[c]] - data_bucket[contracts[c]]['CLOSE'])
            / data_bucket[contracts[c]]['CLOSE'] * 100,
            2
        )
        data_bucket[contracts[c]]['basis_chg'] = round(
            (
                data_bucket[contracts[c]][contracts[c]] / data_bucket[contracts[c]][contracts[c]].shift(1)
                - data_bucket[contracts[c]]['CLOSE'] / data_bucket[contracts[c]]['CLOSE'].shift(1)
            ) * 100, 2
        )
        if c == 1:
            selected = True
        else:
            selected = False
        web.add_yaxis(
            series_name=contracts[c][2:6],
            y_axis=(
                    data_bucket[contracts[c]][contracts[c]]
                    / data_bucket[contracts[c]][data_bucket[contracts[c]][contracts[c]] > 0][contracts[c]].tolist()[0]
            ).round(4),
            label_opts=opts.LabelOpts(is_show=False),
            yaxis_index=0,
            is_selected=selected,
        )

    web.extend_axis(
        yaxis=opts.AxisOpts(
            name='基差',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            position="right",
            # interval=5,
            min_='dataMin',
            max_='dataMax',
        )
    ).extend_axis(
        yaxis=opts.AxisOpts(
            name='基差变化',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            position="right",
            offset=80,
            min_='dataMin',
            max_='dataMax',
            # interval=5
        )
    ).set_global_opts(
        title_opts=opts.TitleOpts(
            title=data_o['TDATE'][len(data_o) - 1]
        ),
        tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        # toolbox_opts=opts.ToolboxOpts(is_show=True),
        xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False),
        yaxis_opts=opts.AxisOpts(
            axislabel_opts=opts.LabelOpts(formatter="{value}"),
            min_='dataMin',
            max_='dataMax',
        ),
        datazoom_opts=[
            opts.DataZoomOpts(range_start=zoom_left, range_end=zoom_right),
            # opts.DataZoomOpts(pos_left="5%", xaxis_index=0),
            # opts.DataZoomOpts(pos_right="5%", xaxis_index=1),
            opts.DataZoomOpts(type_="inside")
        ],
        legend_opts=opts.LegendOpts(
            pos_top="7%",
            pos_left='10%',
            pos_right='10%'
        )
    )

    web2 = Line().add_xaxis(
        xaxis_data=data_o['TDATE']
    )
    web3 = Bar().add_xaxis(
        xaxis_data=data_o['TDATE']
    )
    for c in range(min(len(contracts), 4)):
        if c == 1:
            selected = True
        else:
            selected = False
        web2.add_yaxis(
            series_name=contracts[c][2:6] + '基差',
            y_axis=data_bucket[contracts[c]]['basis'].tolist(),
            label_opts=opts.LabelOpts(is_show=False),
            yaxis_index=1,
            is_selected=False,
        )

        web3.add_yaxis(
            series_name=contracts[c][2:6] + '变化',
            y_axis=data_bucket[contracts[c]]['basis_chg'].tolist(),
            label_opts=opts.LabelOpts(is_show=False),
            yaxis_index=2,
            is_selected=selected,
            # areastyle_opts=opts.AreaStyleOpts(opacity=0.5),
        )

    web.overlap(web2).overlap(web3)

    grid = (
        Grid(init_opts=opts.InitOpts(
            width=str(grid_width) + "px", height=str(grid_height) + "px")
        ).add(
            web, grid_opts=opts.GridOpts(
                pos_right=str(chart_pos_right) + "%",
                pos_left=str(chart_pos_left) + "%",
                pos_top='25%',
            ),
            is_control_axis_index=True
        )
    )
    return grid


def fut_sec_stats_wind(
        start_date=datetime(2010, 1, 1).date(), end_date=datetime.now().date(),
        db_path=sql_write_path_hb['daily'], grid_width=1200, grid_height=600, key='amount', freq=None, to_excel=False
):
    com_data = wind_com_fut_sec_index(start_date=start_date, end_date=end_date, db_path=db_path, key=key, decimal=1)
    index_data = wind_fin_fut_index(start_date=start_date, end_date=end_date, db_path=db_path, key=key, decimal=1)
    data = com_data.merge(index_data, on='t_date')
    columns = data.columns.tolist()
    columns.pop(columns.index('t_date'))

    data = data.set_index(pd.to_datetime(data['t_date']))
    x_axis = data['t_date']
    if freq is not None:
        x_axis = data['t_date'].resample(freq).last()
        data = data.resample(freq).mean()

    if to_excel:
        data.index = pd.to_datetime(data.index).date.tolist()
        return data

    if key.lower() == 'oi_amount':
        name = '期货持仓额(日均)'
        decimal = 100000000
        unit_str = '亿元'
    elif key.lower() == 'volume':
        name = '期货成交量(日均)'
        decimal = 10000
        unit_str = '万手'
    elif key.lower() == 'oi':
        name = '期货持仓量(日均)'
        decimal = 10000
        unit_str = '万手'
    else:
        name = '期货成交额(日均)'
        decimal = 100000000
        unit_str = '亿元'

    web = Line().add_xaxis(xaxis_data=x_axis)

    for i in columns:
        web.add_yaxis(
            series_name=i,
            y_axis=(data[i] / decimal).round(2).tolist(),
            stack='总量',
            label_opts=opts.LabelOpts(is_show=False),
            areastyle_opts=opts.AreaStyleOpts(opacity=0.5),
        )

    web.set_global_opts(
        title_opts=opts.TitleOpts(title=name),
        tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        yaxis_opts=opts.AxisOpts(
            type_="value",
            axislabel_opts=opts.LabelOpts(formatter="{value} " + unit_str),
            axistick_opts=opts.AxisTickOpts(is_show=True),
            splitline_opts=opts.SplitLineOpts(is_show=True),
        ),
        xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False),
        datazoom_opts=[
            opts.DataZoomOpts(
                range_start=0,
                range_end=100,
                # is_show=False,
                type_="inside",
                # xaxis_index=[0, 1],
            ),
            opts.DataZoomOpts(
            #     is_show=True,
            #     xaxis_index=[0, 1],
            #     type_="slider",
            #     pos_top="90%",
            #     range_start=98,
            #     range_end=100,
            ),
        ],
        legend_opts=opts.LegendOpts(
            pos_top="7%",
            pos_left='10%',
            pos_right='10%'
        )
    )
    # web2.set_global_opts(
    #     legend_opts=opts.LegendOpts(
    #         pos_top="7%",
    #         pos_left='10%',
    #         pos_right='20%'
    #     )
    # )
    grid = (
        Grid(
            init_opts=opts.InitOpts(
                width=str(grid_width) + "px",
                height=str(grid_height) + "px"
            ),
        ).add(
            web, grid_opts=opts.GridOpts(
                pos_right="10%",
                pos_left="10%",
                pos_top='20%',
                # pos_bottom='50%'
            ),
            # is_control_axis_index=True
        # ).add(
        #     web2, grid_opts=opts.GridOpts(
        #         pos_right="20%",
        #         pos_left="10%",
        #         pos_top='55%',
        #         pos_bottom='15%'
        #     ),
        )
    )

    return grid


def fin_fut_stats(product, start_date=None, end_date=None, grid_width=1200, grid_height=600, to_excel=False):
    if start_date is None:
        start_date = datetime(2010, 1, 1).date()
    if end_date is None:
        end_date = datetime.now().date()

    fut_data = fin_fut_daily_k_by_contracts(
        contract_list=HSJY_VIRTUAL_CONTRACT['FINANCE'][product].values(),
        start_date=start_date,
        end_date=end_date
    )
    transaction_data = pd.DataFrame(fut_data['data'])
    transaction_data['TRADINGDAY'] = pd.to_datetime(transaction_data['TRADINGDAY']).dt.date.tolist()
    transaction_data = transaction_data.groupby(by='TRADINGDAY').sum()[
        ['TurnoverVolume'.upper(), 'TurnoverValue'.upper(), 'OpenInterest'.upper()]
    ].reset_index()
    if to_excel:
        return transaction_data

    web = Line(
        # init_opts=opts.InitOpts(width="1200px", height="400px")
    ).add_xaxis(
        xaxis_data=transaction_data['TRADINGDAY'].tolist()
    ).add_yaxis(
        series_name='成交量',
        y_axis=list(transaction_data['TurnoverVolume'.upper()].values / 10000),
        linestyle_opts=opts.LineStyleOpts(
            # color="red",
            width=1,
            # type_="dashed"
        ),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=0
    ).extend_axis(
        yaxis=opts.AxisOpts(
            name='持仓量',
            axislabel_opts=opts.LabelOpts(formatter="{value} 万手"),
            position="right",
            # interval=5
        )
    ).extend_axis(
        yaxis=opts.AxisOpts(
            name='成交额',
            axislabel_opts=opts.LabelOpts(formatter="{value} 亿元"),
            position="right",
            offset=80
            # interval=5
        )
    ).set_global_opts(
        title_opts=opts.TitleOpts(
            title=product + ' ' + transaction_data['TRADINGDAY'][len(transaction_data) - 1].strftime('%Y-%m-%d')
        ),
        tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        # toolbox_opts=opts.ToolboxOpts(is_show=True),
        xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False),
        yaxis_opts=opts.AxisOpts(axislabel_opts=opts.LabelOpts(formatter="{value} 万手")),
        datazoom_opts=[
            opts.DataZoomOpts(range_start=0, range_end=100),
            # opts.DataZoomOpts(pos_left="5%", xaxis_index=0),
            # opts.DataZoomOpts(pos_right="5%", xaxis_index=1),
            opts.DataZoomOpts(type_="inside")
        ]
    )

    web2 = Line().add_xaxis(
        xaxis_data=transaction_data['TRADINGDAY'].tolist()
    ).add_yaxis(
        series_name='持仓量',
        y_axis=list(transaction_data['OpenInterest'.upper()].values / 10000),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=1
    )

    web3 = Line().add_xaxis(
        xaxis_data=transaction_data['TRADINGDAY'].tolist()
    ).add_yaxis(
        series_name='成交额',
        y_axis=list(transaction_data['TurnoverValue'.upper()].values / 100000000),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=2
    )

    web.overlap(web2).overlap(web3)

    grid = (
        Grid(init_opts=opts.InitOpts(
            width=str(grid_width) + "px", height=str(grid_height) + "px")
        ).add(
            web, grid_opts=opts.GridOpts(pos_right="20%", pos_left="10%"),
            is_control_axis_index=True
        )
    )

    return grid


def funds_return_stats(
        funds, end_date, lookback_year, grid_width=1200, grid_height=600, pos_top=20, interval=0.5,
        dtype='', fillna=False, freq='w',
        db_path=sql_write_path_hb['work'], cal_path=sql_write_path_hb['daily']
):
    date_end = end_date
    date_start = date_end - timedelta(days=365 * lookback_year)
    table = 'fund_list'

    fund_list = pd.read_sql_query(
        'select * from ' + table + ' where `name` in ' + str(tuple(funds)), create_engine(db_path)
    )

    if dtype.lower() != 'alpha':
        funds_data = load_funds_data(
            fund_list=fund_list,
            first_date=date_start,
            end_date=date_end,
            fillna=fillna,
            db_path=db_path,
            cal_db_path=cal_path,
            freq=freq
        )
    else:
        funds_data = load_funds_alpha(
            fund_list=fund_list,
            first_date=date_start,
            end_date=date_end,
            fillna=fillna,
            db_path=db_path,
            cal_db_path=cal_path,
            freq=freq
        )

    ret_df = ret(data_df=funds_data)
    ret_data = ret_df[funds]
    ret_values = np.reshape(ret_data.values, (1, ret_data.values.shape[0] * ret_data.values.shape[1]))
    ret_values = ret_values[~np.isnan(ret_values)] * 100
    ret_floor = np.floor(min(ret_values)) - 1
    ret_ceil = np.ceil(max(ret_values))
    ret_space = np.linspace(start=ret_floor, stop=ret_ceil, num=int((ret_ceil - ret_floor) / interval + 1))
    dist_df = pd.DataFrame({'zone': ret_space})
    dist_df['label'] = dist_df['zone'].apply(lambda x: str(x) + '% ~ ' + str(x + interval) + '%')
    for i in funds:
        dist_df[i] = 0
        for j in range(1, len(ret_df)):
            fund_ret = ret_df[i][j] * 100
            if np.isnan(fund_ret):
                continue
            fund_ret_index = sum(ret_space <= fund_ret) - 1

            dist_df.loc[fund_ret_index, i] += 1

    web = Bar(
    ).add_xaxis(
        xaxis_data=dist_df['label'].tolist()
    )
    for i in range(len(funds)):
        f_name = funds[i]
        if i == 0:
            sel = True
        else:
            sel = False
        web.add_yaxis(
            series_name=f_name,
            y_axis=dist_df[f_name].tolist(),
            stack='总量',
            label_opts=opts.LabelOpts(is_show=False),
            is_selected=sel,
            # areastyle_opts=opts.AreaStyleOpts(opacity=0.5),
        )

    web.set_global_opts(
        title_opts=opts.TitleOpts(
            title='近' + str(lookback_year) + '年产品周收益分布（平均周收益：' + str(np.mean(ret_values).round(2))
                  + '%）' + funds_data['t_date'].tolist()[-1].strftime('%Y-%m-%d')
        ),
        tooltip_opts=opts.TooltipOpts(
            # trigger="axis", axis_pointer_type="cross"
        ),
        yaxis_opts=opts.AxisOpts(
            type_="value",
            axislabel_opts=opts.LabelOpts(formatter="{value}"),
            axistick_opts=opts.AxisTickOpts(is_show=True),
            splitline_opts=opts.SplitLineOpts(is_show=True),
        ),
        xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False),
        # datazoom_opts=[
        #     opts.DataZoomOpts(
        #         range_start=0,
        #         range_end=100,
        #         # is_show=False,
        #         type_="inside",
        #         # xaxis_index=[0, 1],
        #     ),
        #     opts.DataZoomOpts(
        #         #     is_show=True,
        #         #     xaxis_index=[0, 1],
        #         #     type_="slider",
        #         #     pos_top="90%",
        #         #     range_start=98,
        #         #     range_end=100,
        #     ),
        # ],
        legend_opts=opts.LegendOpts(
            pos_top="7%",
            pos_left='5%',
            pos_right='5%'
        )
    )
    # web2.set_global_opts(
    #     legend_opts=opts.LegendOpts(
    #         pos_top="7%",
    #         pos_left='10%',
    #         pos_right='20%'
    #     )
    # )
    grid = (
        Grid(
            init_opts=opts.InitOpts(
                width=str(grid_width) + "px",
                height=str(grid_height) + "px"
            ),
        ).add(
            web, grid_opts=opts.GridOpts(
                pos_right="10%",
                pos_left="10%",
                pos_top=str(pos_top) + '%',
                # pos_bottom='50%'
            ),
            # is_control_axis_index=True
            # ).add(
            #     web2, grid_opts=opts.GridOpts(
            #         pos_right="20%",
            #         pos_left="10%",
            #         pos_top='55%',
            #         pos_bottom='15%'
            #     ),
        )
    )
    return grid


def com_fut_amt(start_date=None, end_date=None, grid_width=1200, grid_height=600):
    data = pd.DataFrame(fut_com_amount_by_ex(start_date=start_date, end_date=end_date)['data'])
    data['ENDDATE'] = pd.to_datetime(data['ENDDATE']).dt.date.tolist()
    data['SUM(TURNOVER)'] = data['SUM(TURNOVER)'] / 100000000
    data.loc[
        data['ENDDATE'] < datetime(2020, 1, 1).date(), 'SUM(TURNOVER)'
    ] = data.loc[
                data['ENDDATE'] < datetime(2020, 1, 1).date(), 'SUM(TURNOVER)'
            ] / 2
    # ex_list = data['EXCHANGE'].drop_duplicates().tolist()
    # calendar = data[['ENDDATE']].drop_duplicates().reset_index(drop=True)
    # calendar['ENDDATE'] = pd.to_datetime(calendar['ENDDATE']).dt.date.tolist()

    web = Line(
    ).add_xaxis(
        xaxis_data=data['ENDDATE'].tolist()
    )
    web.add_yaxis(
        series_name='AMOUNT',
        y_axis=data['SUM(TURNOVER)'].tolist(),
        # stack='总量',
        label_opts=opts.LabelOpts(is_show=False),
        areastyle_opts=opts.AreaStyleOpts(opacity=0.5),
    )

    # for i in ex_list:
    #     data_e = data[['ENDDATE', 'TURNOVER']].groupby(by='ENDDATE').sum().reset_index()
    #     data_e['TURNOVER'] = data_e['TURNOVER'] / 100000000
    #
    #     web.add_yaxis(
    #         series_name=exchange_dict[i],
    #         y_axis=data_e['TURNOVER'].tolist(),
    #         stack='总量',
    #         label_opts=opts.LabelOpts(is_show=False),
    #         areastyle_opts=opts.AreaStyleOpts(opacity=0.5),
    #     )

    web.set_global_opts(
        title_opts=opts.TitleOpts(title="每日期货成交额(亿元) " + data['ENDDATE'].tolist()[-1].strftime('%Y-%m-%d')),
        tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        yaxis_opts=opts.AxisOpts(
            type_="value",
            axislabel_opts=opts.LabelOpts(formatter="{value} 亿元"),
            axistick_opts=opts.AxisTickOpts(is_show=True),
            splitline_opts=opts.SplitLineOpts(is_show=True),
        ),
        xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False),
        datazoom_opts=[
            opts.DataZoomOpts(
                range_start=0,
                range_end=100,
                # is_show=False,
                type_="inside",
                # xaxis_index=[0, 1],
            ),
            opts.DataZoomOpts(
                #     is_show=True,
                #     xaxis_index=[0, 1],
                #     type_="slider",
                #     pos_top="90%",
                #     range_start=98,
                #     range_end=100,
            ),
        ],
        legend_opts=opts.LegendOpts(
            pos_top="7%",
            pos_left='10%',
            pos_right='10%'
        )
    )
    # web2.set_global_opts(
    #     legend_opts=opts.LegendOpts(
    #         pos_top="7%",
    #         pos_left='10%',
    #         pos_right='20%'
    #     )
    # )
    grid = (
        Grid(
            init_opts=opts.InitOpts(
                width=str(grid_width) + "px",
                height=str(grid_height) + "px"
            ),
        ).add(
            web, grid_opts=opts.GridOpts(
                pos_right="20%",
                pos_left="10%",
                pos_top='20%',
                # pos_bottom='50%'
            ),
            # is_control_axis_index=True
            # ).add(
            #     web2, grid_opts=opts.GridOpts(
            #         pos_right="20%",
            #         pos_left="10%",
            #         pos_top='55%',
            #         pos_bottom='15%'
            #     ),
        )
    )

    return grid


# def chart_ret_with_index(
#         funds, end_date, lookback_year, grid_width=1200, grid_height=600, interval=0.5,
#         dtype='', fillna=False, freq='w',
#         db_path=sql_write_path_hb['work'], cal_path=sql_write_path_hb['daily'], *kwargs
# )


def volatility_stats(data, data_name, width=1200, height=600):
    vol_cone, vol_rolling = rolling_volatility(data=data)

    axis_cross = opts.TooltipOpts(trigger="axis", axis_pointer_type="cross")

    web = Line(
        init_opts=opts.InitOpts(width=str(width) + "px", height=str(height) + "px")
    ).add_xaxis(
        xaxis_data=vol_rolling['t_date'].tolist()
    ).add_yaxis(
        series_name=data_name,
        y_axis=vol_rolling['close'].tolist(),
        label_opts=opts.LabelOpts(is_show=False),
        yaxis_index=0,
        is_selected=False
    ).set_series_opts(
        areastyle_opts=opts.AreaStyleOpts(opacity=0.1),
        label_opts=opts.LabelOpts(is_show=False),
    ).extend_axis(
        yaxis=opts.AxisOpts(
            # name='波动率',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            # position="right",
            # interval=5,
            min_='dataMin',
        )
    ).set_global_opts(
        title_opts=opts.TitleOpts(
            title=vol_rolling['t_date'][0].strftime('%Y/%m/%d')
                  + '~'
                  + vol_rolling['t_date'][len(vol_rolling) - 1].strftime('%Y/%m/%d')
            ,
            # pos_right="5%"
        ),
        tooltip_opts=axis_cross,
        legend_opts=opts.LegendOpts(
            pos_top="6%",
            # pos_right='40%'
        ),
        datazoom_opts=[
            opts.DataZoomOpts(range_start=0, range_end=100),
            # opts.DataZoomOpts(pos_left="5%", xaxis_index=0),
            # opts.DataZoomOpts(pos_right="5%", xaxis_index=1),
            opts.DataZoomOpts(type_="inside")
        ],
        yaxis_opts=opts.AxisOpts(
            # axislabel_opts=opts.LabelOpts(formatter="{value}"),
            min_='dataMin',
        ),
    )

    web_vol = Line().add_xaxis(
        xaxis_data=vol_rolling['t_date'].tolist()
    )
    for i in vol_rolling.columns:
        if '日' not in i:
            continue
        web_vol.add_yaxis(
            series_name=i,
            y_axis=vol_rolling[i].round(2).tolist(),
            # symbol="emptyCircle",
            # is_symbol_show=True,
            label_opts=opts.LabelOpts(is_show=False),
            # is_selected=all_selected,
            yaxis_index=1,
        )

    web.overlap(web_vol)

    return web


def factor_index(
        start_date, end_date, factor, params, sql_path, table, freq='', factor_end='', show_end='日', axis_cross=None
):
    web = None
    index_data = None
    if axis_cross:
        axis_cross = opts.TooltipOpts(trigger="axis", axis_pointer_type="cross")

    for window_days in params:
        index_name = factor + str(window_days) + factor_end
        index_data = pd.read_sql_query(
            'select * from ' + table
            + ' where TDATE<=' + end_date.strftime('%Y%m%d')
            + ' and TDATE>=' + start_date.strftime('%Y%m%d')
            + ' and `FACTOR`=\'' + index_name + '\' order by `TDATE`',
            sql_path
        )
        if freq.upper() in ['W', 'M']:
            index_data = index_data.set_index(pd.to_datetime(index_data['TDATE'])).resample(freq.upper()).last()
            index_data = index_data[index_data['CLOSE'] >= 0]
        if web is None:
            web = Line(
                # init_opts=opts.InitOpts(width="1200px", height="400px")
            ).add_xaxis(
                xaxis_data=index_data['TDATE'].tolist()
            )
        web.add_yaxis(
            series_name=str(window_days) + show_end,
            y_axis=list((index_data['CLOSE'] / index_data['CLOSE'][0]).round(4).values),
            label_opts=opts.LabelOpts(is_show=False),
            # yaxis_index=0
        )

    web.set_global_opts(
        title_opts=opts.TitleOpts(
            title=index_data['TDATE'].tolist()[-1]
        ),
        tooltip_opts=axis_cross,
        # toolbox_opts=opts.ToolboxOpts(is_show=True),
        xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False),
        yaxis_opts=opts.AxisOpts(
            axislabel_opts=opts.LabelOpts(formatter="{value}"),
            min_='dataMin',
        ),
        datazoom_opts=[
            opts.DataZoomOpts(range_start=0, range_end=100),
            # opts.DataZoomOpts(pos_left="5%", xaxis_index=0),
            # opts.DataZoomOpts(pos_right="5%", xaxis_index=1),
            opts.DataZoomOpts(type_="inside")
        ],
        legend_opts=opts.LegendOpts(
            pos_top="7%",
            pos_left='5%',
            pos_right='5%'
        )
    )

    grid = (
        Grid(init_opts=opts.InitOpts(
            width="630px", height="400px")
        ).add(
            web,
            grid_opts=opts.GridOpts(
                pos_right="0%",
                pos_left="10%",
                pos_top="20%",
            ),
            is_control_axis_index=True
        )
    )
    return grid


def market_win(start_date, end_date, freq='D', table='index_win', sql_path=sql_write_path_hb['daily']):
    index_win_data = pd.read_sql_query(
        'select * from ' + table + ' where TDATE<=' + end_date.strftime('%Y%m%d')
        + ' and TDATE>=' + start_date.strftime('%Y%m%d')
        + ' and FREQ=\'' + freq + '\'',
        sql_path
    )

    web = Bar(
        init_opts=opts.InitOpts(width="1200px", height="400px")
    ).add_xaxis(
        xaxis_data=index_win_data['TDATE'].tolist()
    )

    def t000905(x):
        if x == '000905':
            return True
        else:
            return False

    col = ['Q1', 'Q25', 'MEDIAN', 'Q75', 'Q99']
    c = Boxplot()
    c.add_xaxis(index_win_data['TDATE'].tolist())
    c.add_yaxis(
        series_name="个股涨跌分散度",
        # y_axis=c.prepare_data(ret_df_new[funds].values.tolist()),
        y_axis=index_win_data[col].round(2).values.tolist(),
        yaxis_index=1,
        tooltip_opts=opts.TooltipOpts(
            formatter=JsCode(
                """function(param) { return [
                            param.name,
                            '99分位：' + param.data[5],
                            '75分位：' + param.data[4],
                            '中位数：' + param.data[3],
                            '25分位：' + param.data[2],
                            '1分位：' + param.data[1],
                        ].join('<br/>') }"""
            )
        ),
    )

    web.extend_axis(
        yaxis=opts.AxisOpts(
            name='涨跌分散度',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            position="right",
            # interval=5,
            min_=index_win_data[col].round(2).min().min(),
            max_=index_win_data[col].round(2).max().max(),
        )
    ).extend_axis(
        yaxis=opts.AxisOpts(
            name='指数涨跌幅',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            position="right",
            offset=80,
            # interval=5,
            min_=index_win_data[col].round(2).min().min(),
            max_=index_win_data[col].round(2).max().max(),
        )
    )

    index_r = Bar().add_xaxis(
        xaxis_data=index_win_data['TDATE'].tolist()
    )

    for i in ['000300', '000905', '000852']:
        # print(i)
        web.add_yaxis(
            series_name='跑赢' + i,
            y_axis=(index_win_data[i + 'w'] / index_win_data['STKN'] * 100).round(2).tolist(),
            label_opts=opts.LabelOpts(is_show=False),
            yaxis_index=0,
            is_selected=t000905(i),
        )
        index_r.add_yaxis(
            series_name=i,
            y_axis=index_win_data[i].round(2).tolist(),
            label_opts=opts.LabelOpts(is_show=False),
            yaxis_index=2,
            is_selected=t000905(i),
        )

    web.set_global_opts(
        title_opts=opts.TitleOpts(
            title=index_win_data['TDATE'].tolist()[0].strftime('%Y/%m/%d')
                  + ' ~ ' + index_win_data['TDATE'].tolist()[-1].strftime('%Y/%m/%d')
        ),
        tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        # toolbox_opts=opts.ToolboxOpts(is_show=True),
        xaxis_opts=opts.AxisOpts(type_="category", boundary_gap=False),
        yaxis_opts=opts.AxisOpts(
            name='跑赢指数个股占比',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            min_=0,
            max_=100,
        ),
        datazoom_opts=[
            opts.DataZoomOpts(range_start=0, range_end=100),
            # opts.DataZoomOpts(pos_left="5%", xaxis_index=0),
            # opts.DataZoomOpts(pos_right="5%", xaxis_index=1),
            opts.DataZoomOpts(type_="inside")
        ],
        legend_opts=opts.LegendOpts(
            pos_top="7%",
            # pos_left='10%',
            # pos_right='10%'
        )
    )

    return web.overlap(index_r).overlap(c)


def gzb_ts_positions(data, width=800, height=600):
    data['绝对占比'] = data['市值占净值'].abs()
    data_value = data.groupby(['日期'], as_index=False)['绝对占比'].sum().rename(columns={'绝对占比': '总比'})
    data_value_by_sector = data.groupby(['日期', '板块'], as_index=False)['绝对占比'].sum()
    data_value_by_sector = data_value_by_sector.merge(data_value, on='日期', how='left')
    data_value_by_sector['相对占比'] = data_value_by_sector['绝对占比'] / data_value_by_sector['总比'] * 100

    data_value_net_by_sector = data.groupby(['日期', '板块'], as_index=False)['市值占净值'].sum()

    sectors = data['板块'].drop_duplicates().tolist()
    web = Bar().add_xaxis(data_value['日期'].tolist())
    web2 = Bar().add_xaxis(data_value['日期'].tolist())
    web3 = Bar().add_xaxis(data_value['日期'].tolist())
    for i in sectors:
        abs_p = data_value_by_sector[data_value_net_by_sector['板块'] == i][['日期', '绝对占比']]
        abs_p = data_value[['日期']].merge(abs_p, on='日期', how='left')

        rel_p = data_value_by_sector[data_value_net_by_sector['板块'] == i][['日期', '相对占比']]
        rel_p = data_value[['日期']].merge(rel_p, on='日期', how='left')

        net_p = data_value_net_by_sector[data_value_net_by_sector['板块'] == i][['日期', '市值占净值']]
        net_p = data_value[['日期']].merge(net_p, on='日期', how='left')

        web.add_yaxis(
            i,
            abs_p['绝对占比'].round(2).tolist(),
            stack='stack1',
        )
        web2.add_yaxis(
            i,
            rel_p['相对占比'].round(2).tolist(),
            stack='stack2',
        )
        web3.add_yaxis(
            i,
            net_p['市值占净值'].round(2).tolist(),
            stack='stack3',
        )

    web.set_series_opts(
        label_opts=opts.LabelOpts(is_show=False)
    ).set_global_opts(
        legend_opts=opts.LegendOpts(is_show=False),
        yaxis_opts=opts.AxisOpts(
            name='总市值敞口',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            # min_='dataMin',
            # max_='dataMax',
        ),
    )

    web3.set_series_opts(
        label_opts=opts.LabelOpts(is_show=False)
    ).set_global_opts(
        legend_opts=opts.LegendOpts(is_show=False),
        yaxis_opts=opts.AxisOpts(
            name='净市值敞口',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            min_='dataMin',
            max_='dataMax',
        ),
    )

    web2.set_series_opts(
        label_opts=opts.LabelOpts(is_show=False)
    ).set_global_opts(
        title_opts=opts.TitleOpts(title=data['基金名称'][0] + " 持仓分布"),
        legend_opts=opts.LegendOpts(
            orient="vertical",
            pos_top="5%",
            # pos_left='2%',
            pos_right='2%',
            type_='scroll',
        ),
        yaxis_opts=opts.AxisOpts(
            name='总市值占比',
            axislabel_opts=opts.LabelOpts(formatter="{value} %"),
            # min_='dataMin',
            max_=100,
        ),
        datazoom_opts=[
            opts.DataZoomOpts(xaxis_index=[0, 0], range_start=0, range_end=100),
            opts.DataZoomOpts(xaxis_index=[0, 1], range_start=0, range_end=100, type_="inside"),
            opts.DataZoomOpts(xaxis_index=[0, 2], range_start=0, range_end=100, type_="inside")
        ],
    )

    grid = (
        Grid(
            init_opts=opts.InitOpts(
                width=str(width) + "px",
                height=str(height) + "px"
            )
        ).add(
            web2,
            grid_opts=opts.GridOpts(
                pos_right="18%",
                pos_left="8%",
                pos_top="10%",
                pos_bottom="60%"
            ),
            # is_control_axis_index=True
        ).add(
            web,
            grid_opts=opts.GridOpts(
                pos_right="18%",
                pos_left="8%",
                pos_top="50%",
                pos_bottom="35%"
            ),
            # is_control_axis_index=True
        ).add(
            web3,
            grid_opts=opts.GridOpts(
                pos_right="18%",
                pos_left="8%",
                pos_top="75%",
                # pos_bottom=""
            ),
            # is_control_axis_index=True
        )
    )

    return grid


def gzb_xs_position(data, width=800, height=600):
    date = data['日期'][len(data) - 1]
    name = data['基金名称'][0]
    data = data[data['日期'] == date].reset_index(drop=True)
    data['绝对占比'] = data['市值占净值'].abs()
    data_value_by_sector = data.groupby(['板块'], as_index=False)['绝对占比'].sum().rename(
        columns={'绝对占比': '板块市值占比'}
    )
    data = data.merge(data_value_by_sector, on='板块', how='left')
    data = data.sort_values(by=['板块市值占比', '绝对占比'], ascending=False)
    data['科目名称'] = data['科目名称'].apply(lambda x: x + ' 空' if data[data['科目名称'] == x]['市值占净值'].tolist()[0] < 0 else x)
    inner_data = data[['板块', '板块市值占比']].drop_duplicates().reset_index(drop=True)
    inner_data_pair = [
        list(z) for z in zip(inner_data['板块'].tolist(), inner_data['板块市值占比'].round(2).tolist())
    ]

    outer_data = data[['科目名称', '绝对占比']].sort_values(by=['绝对占比'], ascending=False).reset_index(drop=True)
    outer_data_pair = [
        list(z) for z in zip(outer_data['科目名称'].tolist(), outer_data['绝对占比'].round(2).tolist())
    ]

    web = Pie(
        init_opts=opts.InitOpts(
            width=str(width) + "px",
            height=str(height) + "px"
        )
    ).add(
        series_name="板块占比",
        data_pair=inner_data_pair,
        radius=[0, "60%"],
        center=["25%", "50%"],
        label_opts=opts.LabelOpts(
            position="inner",
            # formatter="{a|{a}}{abg|}\n{hr|}\n {b|{b}: }{c}  {per|{d}%}  ",
            # background_color="#eee",
            # border_color="#aaa",
            # border_width=1,
            # border_radius=4,
            # rich={
            #     "a": {"color": "#999", "lineHeight": 22, "align": "center"},
            #     "abg": {
            #         "backgroundColor": "#e3e3e3",
            #         "width": "100%",
            #         "align": "right",
            #         "height": 22,
            #         "borderRadius": [4, 4, 0, 0],
            #     },
            #     "hr": {
            #         "borderColor": "#aaa",
            #         "width": "100%",
            #         "borderWidth": 0.5,
            #         "height": 0,
            #     },
            #     "b": {"fontSize": 12, "lineHeight": 33},
            #     "per": {
            #         "color": "#eee",
            #         "backgroundColor": "#334455",
            #         "padding": [2, 4],
            #         "borderRadius": 2,
            #     },
            # },
        )
    ).add(
        series_name="品种占比",
        radius=[0, "30%"],
        center=["75%", "50%"],
        data_pair=outer_data_pair,
        label_opts=opts.LabelOpts(
            position="outside",
            # formatter="{a|{a}}{abg|}\n{hr|}\n {b|{b}: }{c}  {per|{d}%}  ",
            # background_color="#eee",
            # border_color="#aaa",
            # border_width=1,
            # border_radius=4,
            # rich={
            #     "a": {"color": "#999", "lineHeight": 22, "align": "center"},
            #     "abg": {
            #         "backgroundColor": "#e3e3e3",
            #         "width": "100%",
            #         "align": "right",
            #         "height": 22,
            #         "borderRadius": [4, 4, 0, 0],
            #     },
            #     "hr": {
            #         "borderColor": "#aaa",
            #         "width": "100%",
            #         "borderWidth": 0.5,
            #         "height": 0,
            #     },
            #     "b": {"fontSize": 16, "lineHeight": 33},
            #     "per": {
            #         "color": "#eee",
            #         "backgroundColor": "#334455",
            #         "padding": [2, 4],
            #         "borderRadius": 2,
            #     },
            # },
        ),
    ).set_global_opts(
        title_opts=opts.TitleOpts(title=name + ' ' + date.strftime('%Y/%m/%d') + " 持仓分布"),
        legend_opts=opts.LegendOpts(
            is_show=False,
            pos_left="left",
            orient="vertical"
        )
    ).set_series_opts(
        tooltip_opts=opts.TooltipOpts(
            trigger="item", formatter="{a} <br/>{b}: {c} ({d}%)"
        )
    )

    return web




import numpy as np
import pandas as pd
from hbshare.fe.XZ import db_engine as dbeng
from hbshare.fe.XZ import functionality


util=functionality.Untils()
hbdb=dbeng.HBDB()
fre_map={'M':'2101','Q':'2103','HA':'2106','A':'2201','2A':'2202','3A':'2203'}
localdb = dbeng.PrvFunDB().engine

class Multi_factor_model:

    def __init__(self,factor_list,fre):

        self.factor_list=factor_list
        self.fre=fre

        factor_df=pd.DataFrame()
        count=0
        for factor_name in factor_list:

            raw_factor_data = self.factro_loader(factor_name)

            raw_factor_data['month'] = raw_factor_data['date'].astype(str).str[4:6]

            if (fre == 'Q'):
                raw_factor_data = raw_factor_data.loc[raw_factor_data['month'].isin(['03', '06', '09', '12'])]
            elif (fre == 'HA'):
                raw_factor_data = raw_factor_data.loc[raw_factor_data['month'].isin(['06', '12'])]
            elif (fre == 'A'):
                raw_factor_data = raw_factor_data.loc[raw_factor_data['month'].isin(['12'])]

            if (count == 0):
                factor_df = raw_factor_data.copy()
            else:
                factor_df = pd.merge(factor_df
                                     , raw_factor_data, how='inner', on=['jjdm', 'date'])
            count += 1

        self.raw_factor_data=factor_df


    @staticmethod
    def factro_loader(factor_name):

        if(factor_name=='25%_year'):

            sql="SELECT * from factor_year_rank_quantile".format(factor_name)
            factor_date=pd.read_sql(sql,con=localdb)[['jjdm','25%','date']].rename(columns={'25%':factor_name})

        elif(factor_name=='25%_half_year'):

            sql="SELECT * from factor_half_year_rank_quantile".format(factor_name)
            factor_date=pd.read_sql(sql,con=localdb)[['jjdm','25%','date']].rename(columns={'25%':factor_name})

        elif (factor_name == 'year_logret'):

            sql="select jjdm,year_logret,date from factor_year_ret where year_logret is not null "
            factor_date=pd.read_sql(sql, con=localdb)

        else:
            print('input factor name is not included in the model by now ')
            raise  Exception

        return  factor_date

    def factor_correlation_check(self):

        factor_df=self.raw_factor_data
        factor_df.drop(['jjdm','month'],axis=1,inplace=True)

        corr_res=factor_df.groupby('date').corr()

        return  corr_res.reset_index().groupby('level_1').mean()

    def get_the_combined_factors(self,method,factor_list=None):

        if(factor_list is None):
            factor_list=self.factor_list

        if(method=='avg'):

            factor_df=self.raw_factor_data[['jjdm','date']+factor_list]
            factor_df['new_factor']=factor_df[factor_list].mean(axis=1)

        else:
            print('the input method is not suppoered ')
            raise  Exception


        return factor_df[['jjdm','date','new_factor']]


if __name__ == '__main__':


    mfm=Multi_factor_model(['25%_year','25%_half_year','year_logret'],fre='Q')
    #corr=mfm.factor_correlation_check()




    print('Done')


"""
股票市场成交及换手数据
"""
import pandas as pd
from datetime import datetime
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class StockTrading:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_stock_trading'

    def get_stock_trading_data(self):
        """
        股票市场交易类数据
        """
        # VIX指数
        res = w.wsd("VIX.GI", "close", self.start_date, self.end_date, "")
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch VIX data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        vix_index = data.copy()
        # 成交额数据：两市、300、500、1000及其他
        res = w.wsd("000001.SH,399001.SZ,000300.SH,000905.SH,000852.SH", "amt", self.start_date, self.end_date, "")
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch amt data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns={"000001.SH": "amt_sh", "399001.SZ": "amt_sz", "000300.SH": "amt_300",
                                 "000905.SH": "amt_500", "000852.SH": "amt_1000"}, inplace=True)
            data.eval("amt_other = amt_sh + amt_sz - amt_300 - amt_500 - amt_1000", inplace=True)
            data = (data.set_index('trade_date') / 1e+8).reset_index()
        amt = data.copy()
        # 换手率数据：两市、300、500、1000
        res = w.wsd("000001.SH,399001.SZ,000300.SH,000905.SH,000852.SH", "turn", self.start_date, self.end_date, "")
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch turn data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns={"000001.SH": "turn_sh", "399001.SZ": "turn_sz", "000300.SH": "turn_300",
                                 "000905.SH": "turn_500", "000852.SH": "turn_1000"}, inplace=True)
        turn = data.copy()
        # 主力资金流入数据
        res = w.wsd("000001.SH,399001.SZ", "mfd_inflow_m", self.start_date, self.end_date, "unit=1")
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch inflow data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns={"000001.SH": "inflow_sh", "399001.SZ": "inflow_sz"}, inplace=True)
            data['inflow_sh'] /= 1e+8
            data['inflow_sz'] /= 1e+8
        flow = data.copy()
        all_data = vix_index.merge(amt, on='trade_date', how='outer').merge(turn, on='trade_date', how='outer').merge(
            flow, on='trade_date', how='outer').rename(columns={"VIX.GI": "VIX"})

        return all_data

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_stock_trading_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            delete_duplicate_records(sql_script)
            WriteToDB().write_to_db(data, self.table_name)
        else:
            data = self.get_stock_trading_data()
            sql_script = """
                    create table mac_stock_trading(
                    id int auto_increment primary key,
                    trade_date date not null unique,
                    VIX decimal(5, 2),
                    amt_sh decimal(7, 2),
                    amt_sz decimal(7, 2),
                    amt_300 decimal(7, 2),
                    amt_500 decimal(7, 2),
                    amt_1000 decimal(7, 2),
                    amt_other decimal(7, 2),
                    turn_sh decimal(4, 2),
                    turn_sz decimal(4, 2),
                    turn_300 decimal(4, 2),
                    turn_500 decimal(4, 2),
                    turn_1000 decimal(4, 2),
                    inflow_sh decimal(6, 2),
                    inflow_sz decimal(6, 2)) 
                """
            create_table(self.table_name, sql_script)
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    StockTrading('2021-10-09', '2022-4-29', is_increment=1).get_construct_result()


# -*- coding: utf-8 -*-

import numpy as np
import pandas as pd
import os
import xlwings as xw


# def get_nav(fof_id):
#     from hbshare.fe.xwq.analysis.orm.hbdb import HBDB
#     from datetime import datetime
#     calendar_df = HBDB().read_cal('20000101', datetime.today().strftime('%Y%m%d'))
#     calendar_df = calendar_df.rename(columns={'JYRQ': 'CALENDAR_DATE', 'SFJJ': 'IS_OPEN', 'SFZM': 'IS_WEEK_END', 'SFYM': 'IS_MONTH_END'})
#     calendar_df['CALENDAR_DATE'] = calendar_df['CALENDAR_DATE'].astype(str)
#     calendar_df = calendar_df.sort_values('CALENDAR_DATE')
#     calendar_df['IS_WEEK_END'] = calendar_df['IS_WEEK_END'].fillna(0).astype(int)
#     weekend_list = calendar_df[calendar_df['IS_WEEK_END'] == 1]['CALENDAR_DATE'].unique().tolist()
#
#     file_path = 'D:/Git/hbshare/hbshare/fe/xwq/data/valuation_file/{0}/'.format(fof_id)
#     files = os.listdir(file_path)
#     for file in files:
#         os.rename('{0}{1}'.format(file_path, file), '{0}{1}'.format(file_path, file.split('_')[-1]))
#
#     nav = pd.Series(index=weekend_list)
#     for date in weekend_list:
#         if not os.path.exists('{0}{1}.xlsx'.format(file_path, date)):
#             continue
#         file_date = pd.read_excel('{0}{1}.xlsx'.format(file_path, date))
#         nav.loc[date] = float(file_date.loc[1, 'Unnamed: 3'].split('单位净值:')[-1])
#     nav = nav.dropna()
#     nav.to_excel('D:/Git/hbshare/hbshare/fe/xwq/data/valuation_file/{0}.xlsx'.format(fof_id))
#     return

def check_holding_diff(last_holding, holding):
    """
    检查上期与本期持仓是否变动
    """
    last_holding_list = last_holding['投资标的'].unique().tolist()
    holding_list = holding['科目名称'].unique().tolist()
    diff1 = list(set(last_holding_list) - set(holding_list))
    diff2 = list(set(holding_list) - set(last_holding_list))
    if len(diff1) == 0 and len(diff2) == 0 and len(last_holding_list) == len(holding_list):
        print('holding not changed!')
        diff, remove, add = False, [], []
    else:
        print('holding changed!')
        diff, remove, add = True, diff1, diff2
    return diff, remove, add


def reform_table(last_valuation_table, valuation_table, holding_info):
    # 上期持仓数据
    last_valuation_date = last_valuation_table[last_valuation_table['返回'] == '估值时间'].iloc[0].tolist()[1]
    if len(valuation_table[valuation_table['估值表数据'] == '周初单位净值:']) > 0:
        last_valuation_nav = float(valuation_table[valuation_table['估值表数据'] == '周初单位净值:'].iloc[0].tolist()[1])
    else:
        last_valuation_nav = float(valuation_table[valuation_table['估值表数据'] == '昨日单位净值'].iloc[0].tolist()[1])
    nav = last_valuation_table[['净值日期', '单位净值']]
    nav['净值日期'] = nav['净值日期'].apply(lambda x: str(int(x)) if not np.isnan(x) else x)
    last_valuation_date_str = last_valuation_date.strftime('%Y/%m/%d') if type(last_valuation_date) != str else last_valuation_date
    nav = nav[nav['净值日期'] <= last_valuation_date_str.replace('/', '')].reset_index().drop('index', axis=1)
    last_holding_start_index = last_valuation_table[last_valuation_table['返回'] == '总净值'].index[0]
    last_holding_end_index = last_valuation_table[last_valuation_table['返回'] == '可用头寸'].index[0]
    last_holding = last_valuation_table.iloc[last_holding_start_index + 1: last_holding_end_index]
    last_holding.columns = last_valuation_table.iloc[last_holding_start_index - 1].tolist()
    last_holding = last_holding.dropna(subset=['投资标的'])
    # 本期公募非债持仓数据
    valuation_date = valuation_table['估值表数据'].iloc[1].split(':')[-1]
    valuation_date = '{}/{}/{}'.format(valuation_date[:4], valuation_date[4:6], valuation_date[6:])
    valuation_nav = float(valuation_table['Unnamed: 3'].iloc[1].split(':')[-1])
    valuation_table_columns = valuation_table.iloc[2].tolist()
    valuation_table = valuation_table.iloc[3:].reset_index().drop('index', axis=1)
    valuation_table.columns = valuation_table_columns
    holding_fund = valuation_table[(valuation_table['科目代码'] > '11050201') & (valuation_table['科目代码'] < '11050299')]
    holding_fund = holding_fund.loc[~holding_fund['科目名称'].str.contains('债')]
    # 本期私募持仓数据
    holding_pfund = valuation_table[(valuation_table['科目代码'] > '11090601') & (valuation_table['科目代码'] < '11090699')]
    # 本期持仓数据
    holding = pd.concat([holding_fund, holding_pfund])
    # 检查上期与本期持仓是否变动：
    # 若持仓无变动，则本期持仓按照上期持仓排序展示；
    # 若持仓有变动，则本期持仓按照上期持仓排序展示，且将新增持仓部分排至末尾
    diff, remove, add = check_holding_diff(last_holding, holding)

    holding = holding[['科目名称', '市值', '市值占净值%', '市价']]
    holding.columns = ['投资标的', '市值', '仓位%', '市价']
    holding = last_holding[['投资标的', '市价']].rename(columns={'市价': '上一次市价'}).merge(holding, on=['投资标的'], how='outer')
    holding['市价'] = holding['市价'].astype(float)
    holding['上一次市价'] = holding['上一次市价'].astype(float)
    holding['区间涨跌幅'] = holding['市价'] / holding['上一次市价'] - 1.0
    holding = holding[['投资标的', '市值', '仓位%', '市价', '上一次市价', '区间涨跌幅']]
    holding = holding.dropna(subset=['投资标的'])
    if len(valuation_table.loc[valuation_table['科目代码'] == '基金资产净值:', '市值']) > 0:
        holding.loc[holding.shape[0]] = ['总净值', valuation_table.loc[valuation_table['科目代码'] == '基金资产净值:', '市值'].values[0],
                                         np.nan, valuation_nav, last_valuation_nav, valuation_nav / last_valuation_nav - 1.0]
    if len(valuation_table.loc[valuation_table['科目代码'] == '1002', '市值']) > 0:
        holding.loc[holding.shape[0]] = ['可用头寸', valuation_table.loc[valuation_table['科目代码'] == '1002', '市值'].values[0],
                                         valuation_table.loc[valuation_table['科目代码'] == '1002', '市值占净值%'].values[0], np.nan, np.nan, np.nan]
    rank_list = last_holding['投资标的'].tolist() + add if len(add) != 0 else last_holding['投资标的'].tolist()
    rank_list = ['总净值'] + rank_list + ['可用头寸']
    holding['投资标的'] = holding['投资标的'].astype('category')
    holding['投资标的'].cat.reorder_categories(rank_list, inplace=True)
    holding = holding.sort_values('投资标的')
    if len(valuation_table.loc[valuation_table['科目代码'] == '2206', '市值']) > 0:
        holding.loc[holding.shape[0]] = ['应付管理人报酬', valuation_table.loc[valuation_table['科目代码'] == '2206', '市值'].values[0] * (-1.0),
                                         valuation_table.loc[valuation_table['科目代码'] == '2206', '市值占净值%'].values[0] * (-1.0), np.nan, np.nan, np.nan]
    if len(valuation_table.loc[valuation_table['科目代码'] == '3003', '市值']) > 0:
        holding.loc[holding.shape[0]] = ['证券清算款', valuation_table.loc[valuation_table['科目代码'] == '3003', '市值'].values[0],
                                         valuation_table.loc[valuation_table['科目代码'] == '3003', '市值占净值%'].values[0], np.nan, np.nan, np.nan]
    holding = holding.merge(holding_info, on=['投资标的'], how='left')

    nav.loc[nav.shape[0]] = [valuation_date.replace('/', ''), valuation_nav]
    nav['收益率'] = nav['单位净值'].pct_change()
    perform = pd.DataFrame(index=['年化收益率', '年化波动率', '夏普', '最大回撤', 'calmar比率', '投资胜率', '平均损益比', '估值时间'], columns=['指标值'])
    perform.loc['年化收益率', '指标值'] = (nav['单位净值'].iloc[-1] / nav['单位净值'].iloc[0]) ** (52.0 / len(nav)) - 1.0
    perform.loc['年化波动率', '指标值'] = np.std(nav['收益率'].dropna(), ddof=1) * np.sqrt(52)
    perform.loc['夏普', '指标值'] = (perform.loc['年化收益率', '指标值'] - 0.03) / perform.loc['年化波动率', '指标值']
    perform.loc['最大回撤', '指标值'] = max(
        [(min(nav['单位净值'].iloc[i:]) / nav['单位净值'].iloc[i] - 1.0) * (-1.0) for i in range(len(nav['单位净值']))])
    perform.loc['calmar比率', '指标值'] = perform.loc['年化收益率', '指标值'] / perform.loc['最大回撤', '指标值']
    perform.loc['投资胜率', '指标值'] = len(nav[nav['收益率'] >= 0]) / float(len(nav.dropna()))
    perform.loc['平均损益比', '指标值'] = nav[nav['收益率'] >= 0]['收益率'].mean() / nav[nav['收益率'] < 0]['收益率'].mean() * (-1.0)
    perform.loc['估值时间', '指标值'] = valuation_date

    # reformed_valuation_table['Unnamed: 2'].iloc[3:11] = perform['指标值']
    # start_index = reformed_valuation_table[reformed_valuation_table['返回'] == '总净值'].index[0]
    # end_index = reformed_valuation_table[reformed_valuation_table['返回'] == '可用头寸'].index[0]
    # holding.columns = list(reformed_valuation_table.columns)[:22]
    # reformed_valuation_table_left = pd.concat([reformed_valuation_table.iloc[:start_index, :22], holding,
    #                                            reformed_valuation_table.iloc[end_index + 1:, :22]]).reset_index().drop('index', axis=1)
    # reformed_valuation_table_right = reformed_valuation_table.iloc[:, 22:]
    # reformed_valuation_table = reformed_valuation_table_left.merge(reformed_valuation_table_right, left_index=True, right_index=True, how='left')
    # reformed_valuation_table.loc[reformed_valuation_table.shape[0], ['净值日期', '单位净值', '收益率']] = nav.iloc[-1]

    perform = perform.reset_index()
    perform.columns = ['投资标的', '市值']
    holding.loc[holding.shape[0]] = [np.nan] * holding.shape[1]
    holding.loc[holding.shape[0]] = ['业绩描述'] + [np.nan] * (holding.shape[1] - 1)
    reformed_valuation_table = pd.concat([holding, perform])
    reformed_valuation_table = reformed_valuation_table.reset_index().drop('index', axis=1)
    reformed_valuation_table = pd.concat([reformed_valuation_table, nav[['净值日期', '单位净值']]], axis=1)
    return reformed_valuation_table


def get_reformed_valuation_table(idx, fof_id, table_path):
    """
    获取基金估值表
    """
    valuation_table = pd.DataFrame()
    for f_name in os.listdir(table_path):
        items = f_name.split('_')
        if items[0] == fof_id:
            valuation_table = pd.read_excel(table_path + f_name)
    last_valuation_table = pd.read_excel(table_path + '总表-量化FOF产品统计汇总.xlsx', sheet_name=str(idx + 1)).drop('Unnamed: 0', axis=1)
    holding_info = pd.read_excel(table_path + '投资标的信息.xlsx').drop('Unnamed: 0', axis=1)
    reformed_valuation_table = reform_table(last_valuation_table, valuation_table, holding_info)
    return reformed_valuation_table


def main(table_path):
    """
    主函数
    """
    excel_writer = pd.ExcelWriter(table_path + '总表-量化FOF产品统计汇总-new.xlsx', engine='openpyxl')
    # 产品代码list，其中产品的顺序与总表中各产品顺序一致
    fof_id_list = ['S21582', 'SCS026', 'SJ4683', 'SGR135', 'SSL554', 'SNX811']
    for idx, fof_id in enumerate(fof_id_list):
        reformed_valuation_table = get_reformed_valuation_table(idx, fof_id, table_path)
        reformed_valuation_table.to_excel(excel_writer, sheet_name=str(idx + 1), index=False)
    excel_writer.close()
    return


if __name__ == '__main__':
    # 注意事项：
    # 每周必操作
    # 1. 创建table_path路径，将各产品的估值表文件存放在此路径，输出的结果文件【总表-量化FOF产品统计汇总-new.xlsx】也存放在此路径
    # 2. 每周五从ams下载上一周五估值表至指定路径table_path，并保证各产品只有上一周五估值表数据
    # 3. 每周五将上期总表放至指定路径table_path，并去掉时间后缀，即文件名为【总表-量化FOF产品统计汇总.xlsx】
    # 调仓时操作
    # 4. 若本期与上期相比持仓有变动（新增），更新【投资标的信息.xlsx】，此文件也放在table_path路径下
    # 使用前测试
    # 5. 不同版本pandas（目前版本是1.4.1）读取excel文件格式可能不一样，测试后看是否需要调整

    # todo:
    # 1. 估值表数据自动获取（目前无法实现）
    # 2. 年化收益率、年化波动率、夏普、calmar比率这几个指标的计算细节
    # 3. 更改excel表格，但需保留格式

    # # fof净值获取
    # fof_id_list = ['S21582', 'SCS026', 'SJ4683', 'SGR135', 'SSL554', 'SNX811']
    # for fof_id in fof_id_list:
    #     get_nav(fof_id)

    table_path = 'D:/Git/hbshare/hbshare/fe/xwq/data/valuation/'  # 修改为自己创建的路径
    main(table_path)


"""
资配指标的更新程序
"""
from WindPy import w
from hbshare.quant.Kevin.asset_allocation.macro_index.currency import Currency, CurrencyShibor
from hbshare.quant.Kevin.asset_allocation.macro_index.credit import Credit
from hbshare.quant.Kevin.asset_allocation.macro_index.inflation import Inflation
from hbshare.quant.Kevin.asset_allocation.macro_index.eco_increase import EconomyIncrease
from hbshare.quant.Kevin.asset_allocation.macro_index.rates_and_price import RatesAndPrice
from hbshare.quant.Kevin.asset_allocation.macro_index.processed_market_index import MarketIndex
from hbshare.quant.Kevin.asset_allocation.macro_index.stock_market import StockValuation, DividendRatio
from hbshare.quant.Kevin.asset_allocation.macro_index.stock_cash_flow import StockCashFlow
from hbshare.quant.Kevin.asset_allocation.macro_index.stock_trading import StockTrading
from hbshare.quant.Kevin.asset_allocation.macro_index.stock_cross_section_vol import StockCrossSectionVol
from hbshare.quant.Kevin.asset_allocation.macro_index.stock_time_series_vol import StockTimeSeriesVol
from hbshare.quant.Kevin.asset_allocation.macro_index.stock_trading_cr import TradingCRNCalculator
from hbshare.quant.Kevin.asset_allocation.macro_index.stock_style_momentum import StyleFactorMomentum
from hbshare.quant.Kevin.asset_allocation.macro_index.stock_style_spread import StyleFactorSpread
from hbshare.quant.Kevin.asset_allocation.macro_index.treasury_yield import TreasuryYield
from hbshare.quant.Kevin.asset_allocation.macro_index.bond_trading import BondTrading
from hbshare.quant.Kevin.asset_allocation.macro_index.future_data import FutureMarketIndex, FuturePrice
from hbshare.quant.Kevin.quant_room.IndexActiveStyleAnalyse import EquityIndexStyleCalculator
from hbshare.quant.Kevin.convertible_bond.CBMarketIV import CBMarketIV, CBMarketPrice, OptionIV
from hbshare.quant.Kevin.asset_allocation.macro_index.stock_share_price_index_futures import StockSharePriceIndex
from hbshare.quant.Kevin.asset_allocation.macro_index.stock_concept import StockConception

w.start()


def update_weekly(start_date, end_date):
    """
    周度更新宏观数据：mac_curr, mac_curr_shibor, mac_credit, mac_inflation, mac_eco_increase,
                  mac_rates_and_price, mac_market_index
    """
    # 宏观-货币类
    Currency(start_date, end_date).get_construct_result()
    CurrencyShibor(start_date, end_date).get_construct_result()
    # 宏观-信用类
    Credit(start_date, end_date).get_construct_result()
    # 宏观-通胀类
    Inflation(start_date, end_date).get_construct_result()
    # 宏观-经济增长类
    EconomyIncrease(start_date, end_date).get_construct_result()
    # 宏观-汇率和价格类
    RatesAndPrice(start_date, end_date).get_construct_result()
    # 宏观-处理过的指数数据
    MarketIndex(start_date, end_date).get_construct_result()


def update_monthly(start_date, end_date):
    """
    月度更新风格因子动量&离散度
    """
    # 股票风格因子：因子动量 & 因子离散度
    StyleFactorMomentum(start_date, end_date).get_construct_result()
    StyleFactorSpread(start_date, end_date).get_construct_result()
    # 指数风格暴露
    start_date = start_date.replace('-', '')
    end_date = end_date.replace('-', '')
    EquityIndexStyleCalculator(start_date, end_date, benchmark_id='000300').get_construct_result()
    EquityIndexStyleCalculator(start_date, end_date, benchmark_id='000905').get_construct_result()
    EquityIndexStyleCalculator(start_date, end_date, benchmark_id='000852').get_construct_result()
    # 个股概念标签
    StockConception(start_date, end_date).get_construct_result()


def update_daily(start_date, end_date):
    """
    日度更新股票市场微观数据
    """
    # 股票-PE
    StockValuation(start_date, end_date).get_construct_result()
    DividendRatio(start_date, end_date).get_construct_result()
    # 股票-资金流向
    StockCashFlow(start_date, end_date).get_construct_result()
    # 股票-成交额&换手率
    StockTrading(start_date, end_date).get_construct_result()
    # 股票-截面波动率
    StockCrossSectionVol(start_date, end_date).get_construct_result()
    # 股票-时序波动率
    StockTimeSeriesVol(start_date, end_date).get_construct_result()
    # 股票-成交集中度
    TradingCRNCalculator(start_date, end_date).get_construct_result()
    # 债券-国债收益率
    TreasuryYield(start_date, end_date).get_construct_result()
    # 债券-质押式回购成交量
    BondTrading(start_date, end_date).get_construct_result()
    # 期货商品指数数据
    FutureMarketIndex(start_date, end_date).get_construct_result()
    FuturePrice(start_date, end_date).get_construct_result()
    # 可转债市场隐含波动率
    CBMarketIV(start_date.replace('-', ''), end_date.replace('-', '')).get_construct_result()
    CBMarketPrice(start_date.replace('-', ''), end_date.replace('-', '')).get_construct_result()
    OptionIV(start_date.replace('-', ''), end_date.replace('-', '')).get_construct_result()
    # 基差数据
    StockSharePriceIndex(start_date, end_date).get_construct_result()


if __name__ == '__main__':
    update_daily(start_date='2022-08-22', end_date='2022-08-22')
    # update_monthly('2022-06-10', '2022-07-31')
    # update_weekly(start_date='2021-09-01', end_date='2022-07-31')




"""
净值归因模型
"""
import numpy as np
import pandas as pd
import statsmodels.api as sm
from datetime import datetime, timedelta
from hbshare.fe.common.util.verifier import verify_type
from hbshare.fe.common.util.exception import InputParameterError, InputParameterWarning
from hbshare.fe.common.util.logger import logger
from hbshare.fe.xwq.analysis.orm.http_request import Curl_Request
from hbshare.fe.xwq.analysis.orm.hbdb import HBDB


def get_calendar_df():
    calendar_df = HBDB().read_cal('19900101', datetime.today().strftime('%Y%m%d'))
    calendar_df = calendar_df.rename(columns={'JYRQ': 'CALENDAR_DATE', 'SFJJ': 'IS_OPEN', 'SFZM': 'IS_WEEK_END', 'SFYM': 'IS_MONTH_END'})
    calendar_df['CALENDAR_DATE'] = calendar_df['CALENDAR_DATE'].astype(str)
    calendar_df = calendar_df.sort_values('CALENDAR_DATE')
    calendar_df['IS_OPEN'] = calendar_df['IS_OPEN'].astype(int).replace({0: 1, 1: 0})
    calendar_df['IS_WEEK_END'] = calendar_df['IS_WEEK_END'].fillna(0).astype(int)
    calendar_df['IS_MONTH_END'] = calendar_df['IS_MONTH_END'].fillna(0).astype(int)
    calendar_df['MONTH'] = calendar_df['CALENDAR_DATE'].apply(lambda x: x[4:6])
    calendar_df['MONTH_DAY'] = calendar_df['CALENDAR_DATE'].apply(lambda x: x[4:])
    calendar_df['IS_QUARTER_END'] = np.where((calendar_df['IS_MONTH_END'] == 1) & (calendar_df['MONTH'].isin(['03', '06', '09', '12'])), 1, 0)
    calendar_df['IS_QUARTER_END'] = calendar_df['IS_QUARTER_END'].astype(int)
    calendar_df['IS_SEASON_END'] = np.where(calendar_df['MONTH_DAY'].isin(['0331', '0630', '0930', '1231']), 1, 0)
    calendar_df['IS_SEASON_END'] = calendar_df['IS_SEASON_END'].astype(int)
    trade_cal = calendar_df[calendar_df['IS_OPEN'] == 1]
    trade_cal['TRADE_DATE'] = trade_cal['CALENDAR_DATE']
    calendar_df = calendar_df.merge(trade_cal[['CALENDAR_DATE', 'TRADE_DATE']], on=['CALENDAR_DATE'], how='left')
    calendar_df['TRADE_DATE'] = calendar_df['TRADE_DATE'].fillna(method='ffill')
    calendar_df = calendar_df[['CALENDAR_DATE', 'IS_OPEN', 'IS_WEEK_END', 'IS_MONTH_END', 'IS_QUARTER_END', 'IS_SEASON_END', 'TRADE_DATE']]
    backup_df = calendar_df[['CALENDAR_DATE']]
    backup_df['近三月_init'] = backup_df['CALENDAR_DATE'].apply(lambda x: '{0}{1}{2}'.format(x[:4], str(int(x[4:6]) - 3).zfill(2), x[6:]) if int(x[4:6]) - 3 > 0 else '{0}{1}{2}'.format(str(int(x[:4]) - 1), str(int(x[4:6]) - 3 + 12).zfill(2), x[6:]))
    backup_df['近六月_init'] = backup_df['CALENDAR_DATE'].apply(lambda x: '{0}{1}{2}'.format(x[:4], str(int(x[4:6]) - 6).zfill(2), x[6:]) if int(x[4:6]) - 6 > 0 else '{0}{1}{2}'.format(str(int(x[:4]) - 1), str(int(x[4:6]) - 6 + 12).zfill(2), x[6:]))
    backup_df['近一年_init'] = backup_df['CALENDAR_DATE'].apply(lambda x: '{0}{1}'.format(str(int(x[:4]) - 1), x[4:]))
    backup_df['近两年_init'] = backup_df['CALENDAR_DATE'].apply(lambda x: '{0}{1}'.format(str(int(x[:4]) - 2), x[4:]))
    backup_df['近三年_init'] = backup_df['CALENDAR_DATE'].apply(lambda x: '{0}{1}'.format(str(int(x[:4]) - 3), x[4:]))
    backup_df['近五年_init'] = backup_df['CALENDAR_DATE'].apply(lambda x: '{0}{1}'.format(str(int(x[:4]) - 5), x[4:]))
    backup_df = backup_df.merge(trade_cal[['TRADE_DATE']], left_on=['近三月_init'], right_on=['TRADE_DATE'], how='left').rename(columns={'TRADE_DATE': '近三月'})
    backup_df = backup_df.merge(trade_cal[['TRADE_DATE']], left_on=['近六月_init'], right_on=['TRADE_DATE'], how='left').rename(columns={'TRADE_DATE': '近六月'})
    backup_df = backup_df.merge(trade_cal[['TRADE_DATE']], left_on=['近一年_init'], right_on=['TRADE_DATE'], how='left').rename(columns={'TRADE_DATE': '近一年'})
    backup_df = backup_df.merge(trade_cal[['TRADE_DATE']], left_on=['近两年_init'], right_on=['TRADE_DATE'], how='left').rename(columns={'TRADE_DATE': '近两年'})
    backup_df = backup_df.merge(trade_cal[['TRADE_DATE']], left_on=['近三年_init'], right_on=['TRADE_DATE'], how='left').rename(columns={'TRADE_DATE': '近三年'})
    backup_df = backup_df.merge(trade_cal[['TRADE_DATE']], left_on=['近五年_init'], right_on=['TRADE_DATE'], how='left').rename(columns={'TRADE_DATE': '近五年'})
    backup_df = backup_df.fillna(method='ffill')
    calendar_df = calendar_df.merge(backup_df[['CALENDAR_DATE', '近三月', '近六月', '近一年', '近两年', '近三年', '近五年']], on=['CALENDAR_DATE'], how='left')
    return calendar_df


def get_factor_data_df():
    index_list = ['CBA00301', 'CBA00801', 'CBA00701', 'CBA04201', 'CBA02501', 'CBA03801', 'H11025', '000832', '000906']
    factor_data_df = HBDB().read_index_daily_k_given_date_and_indexs('19900101', index_list)
    factor_data_df = factor_data_df.rename(columns={'zqdm': 'INDEX_CODE', 'jyrq': 'TRADE_DATE', 'spjg': 'CLOSE_INDEX'})
    factor_data_df['TRADE_DATE'] = factor_data_df['TRADE_DATE'].astype(str)
    factor_data_df = factor_data_df.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
    factor_data_df = factor_data_df.sort_index()
    duration_index_list = ['CBA00801', 'CBA00701', 'CBA04201', 'CBA02501', 'CBA03801']
    duration_data_df = Curl_Request().get_duration_between_dates('19900101', datetime.today().strftime('%Y%m%d'))
    duration_data_df = duration_data_df.rename(columns={'zqdm': 'INDEX_CODE', 'jyrq': 'TRADE_DATE', 'jq': 'DURATION'})
    duration_data_df = duration_data_df[duration_data_df['INDEX_CODE'].isin(duration_index_list)]
    duration_data_df['TRADE_DATE'] = duration_data_df['TRADE_DATE'].astype(str)
    duration_data_df = duration_data_df.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='DURATION')
    duration_data_df = duration_data_df.sort_index()
    duration_data_df.columns = [col + '_JQ' for col in duration_data_df.columns]
    factor_data_df = factor_data_df.merge(duration_data_df, left_index=True, right_index=True, how='left')
    return factor_data_df


class BondCampisiAttribution:
    def __init__(self, calendar_df, factor_data_df, fund_id, fund_type, start_date, end_date, frequency, backup_period='整个季度', convertible=True, equity=True):
        """
        :param fund_id: 基金代码
        :param start_date: 归因的起始时间
        :param end_date: 归因的结束时间
        :param frequency: 归因频率
        """
        self.calendar_df = calendar_df
        self.factor_data_df = factor_data_df
        self.fund_id = fund_id
        self.fund_type = fund_type
        self.frequency = frequency
        self.backup_period = '整个季度' if self.fund_type == 'mutual' and self.frequency == 'season' else '两个季度' if self.fund_type == 'private' and self.frequency == 'season' else backup_period
        self.backup_period_days = {'整个季度': 200, '两个季度': 200, '近三月': 200, '近六月': 200, '近一年': 400, '近两年': 800, '近三年': 1200, '近五年': 2000}
        self.start_date_backup = (datetime.strptime(start_date, '%Y%m%d') - timedelta(self.backup_period_days[self.backup_period])).strftime('%Y%m%d')
        self.start_date = start_date
        self.end_date = end_date
        self.all_factor_list = ['F_level', 'F_slope', 'F_credit', 'F_default', 'F_currency', 'F_convertible', 'F_equity']
        self.convertible = convertible
        self.equity = equity
        self.deposit = 0.015 / 365.0 if self.fund_type == 'mutual' else 0.015 / 52.0
        self._verify_input_param()
        self._load_data()
        self._init_bond_campisi_attribution_data()

    def _verify_input_param(self):
        verify_type(self.fund_id, 'fund_id', str)
        verify_type(self.fund_type, 'fund_type', str)
        verify_type(self.start_date, 'start_date', str)
        verify_type(self.end_date, 'end_date', str)
        verify_type(self.frequency, 'frequency', str)
        verify_type(self.convertible, 'convertible', bool)
        verify_type(self.equity, 'equity', bool)
        if self.fund_type not in ['mutual', 'private']:
            msg = "fund_type not supported, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if self.frequency not in ['day', 'week', 'month', 'quarter', 'season']:
            msg = "frequency not supported, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if self.backup_period not in ['近三月', '近六月', '近一年', '近两年', '近三年', '近五年', '整个季度', '两个季度']:
            msg = "backup_period not supported, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if self.fund_type == 'mutual' and self.frequency == 'season' and self.backup_period != '整个季度':
            msg = "fund_type, frequency and backup_period not matched, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if self.fund_type == 'mutual' and self.frequency != 'season' and self.backup_period in ('整个季度', '两个季度'):
            msg = "fund_type, frequency and backup_period not matched, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if self.fund_type == 'private' and self.frequency == 'season' and self.backup_period != '两个季度':
            msg = "fund_type, frequency and backup_period not matched, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)
        if self.fund_type == 'private' and self.frequency != 'season' and self.backup_period in ('整个季度', '两个季度'):
            msg = "fund_type, frequency and backup_period not matched, check your input"
            logger.error(msg)
            raise InputParameterError(message=msg)

    def _load_data(self):
        self.calendar_df = self.calendar_df[self.calendar_df['CALENDAR_DATE'] <= self.end_date]

        if self.fund_type == 'mutual':
            self.fund_nav_series = HBDB().read_fund_cumret_given_code(self.fund_id, self.start_date_backup, self.end_date)
            if len(self.fund_nav_series) == 0:
                self.fund_nav_series = pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'CUM_RET'])
            self.fund_nav_series['TRADE_DATE'] = self.fund_nav_series['TRADE_DATE'].astype(str)
            self.fund_nav_series = self.fund_nav_series.sort_values('TRADE_DATE')
            self.fund_nav_series['ADJ_NAV'] = 0.01 * self.fund_nav_series['CUM_RET'] + 1
            self.fund_nav_series = self.fund_nav_series.set_index('TRADE_DATE')
            self.fund_nav_series = self.fund_nav_series['ADJ_NAV']
        else:
            self.fund_nav_series = HBDB().read_private_fund_cumret_given_code(self.fund_id, self.start_date_backup, self.end_date)
            if len(self.fund_nav_series) == 0:
                self.fund_nav_series = pd.DataFrame(columns=['FUND_CODE', 'TRADE_DATE', 'ADJ_NAV'])
            self.fund_nav_series['TRADE_DATE'] = self.fund_nav_series['TRADE_DATE'].astype(str)
            self.fund_nav_series = self.fund_nav_series.sort_values('TRADE_DATE')
            self.fund_nav_series = self.fund_nav_series.set_index('TRADE_DATE')
            self.fund_nav_series = self.fund_nav_series['ADJ_NAV']

    def get_trading_day_list(self, nav_frequency):
        if nav_frequency == 'day':
            trading_day_list = sorted(list(self.calendar_df[self.calendar_df['IS_OPEN'] == 1]['CALENDAR_DATE']))
        elif nav_frequency == 'week':
            trading_day_list = sorted(list(self.calendar_df[self.calendar_df['IS_WEEK_END'] == 1]['CALENDAR_DATE']))
        elif nav_frequency == 'month':
            trading_day_list = sorted(list(self.calendar_df[self.calendar_df['IS_MONTH_END'] == 1]['CALENDAR_DATE']))
        elif nav_frequency == 'quarter':
            trading_day_list = sorted(list(self.calendar_df[self.calendar_df['IS_QUARTER_END'] == 1]['CALENDAR_DATE']))
        elif nav_frequency == 'season':
            trading_day_list = sorted(list(self.calendar_df[self.calendar_df['IS_SEASON_END'] == 1]['CALENDAR_DATE']))
        else:
            trading_day_list = []
        return trading_day_list

    def _init_bond_campisi_attribution_data(self):
        compute_list = self.get_trading_day_list(self.frequency)
        compute_list = [date for date in compute_list if (date >= self.start_date) and (date <= self.end_date)]
        if self.fund_type == 'mutual':
            date_list = self.get_trading_day_list('day')
        else:
            date_list = self.get_trading_day_list('week')
        if len(compute_list) < 1:
            msg = "There is no complete {0} interval during start_date and end_date, check your input".format(self.frequency)
            logger.error(msg)
            raise InputParameterWarning(message=msg)

        self.factor_data_df = self.factor_data_df[(self.factor_data_df.index >= self.start_date_backup) & (self.factor_data_df.index <= self.end_date)]
        self.factor_data_df = self.factor_data_df.reindex(date_list).interpolate().dropna().sort_index()
        self.factor_data_df[['CBA00301', 'CBA00801', 'CBA00701', 'CBA04201', 'CBA02501', 'CBA03801', 'H11025', '000832', '000906']] = self.factor_data_df[['CBA00301', 'CBA00801', 'CBA00701', 'CBA04201', 'CBA02501', 'CBA03801', 'H11025', '000832', '000906']].pct_change()
        self.factor_data_df['F_level'] = self.factor_data_df['CBA00301'] - self.deposit
        self.factor_data_df['F_slope'] = self.factor_data_df['CBA00701_JQ'] / (self.factor_data_df['CBA00701_JQ'] - self.factor_data_df['CBA00801_JQ']) * self.factor_data_df['CBA00801'] \
                                       + self.factor_data_df['CBA00801_JQ'] / (self.factor_data_df['CBA00801_JQ'] - self.factor_data_df['CBA00701_JQ']) * self.factor_data_df['CBA00701']
        self.factor_data_df['F_credit'] = self.factor_data_df['CBA02501_JQ'] / (self.factor_data_df['CBA02501_JQ'] - self.factor_data_df['CBA04201_JQ']) * self.factor_data_df['CBA04201'] \
                                        + self.factor_data_df['CBA04201_JQ'] / (self.factor_data_df['CBA04201_JQ'] - self.factor_data_df['CBA02501_JQ']) * self.factor_data_df['CBA02501']
        self.factor_data_df['F_default'] = self.factor_data_df['CBA04201_JQ'] / (self.factor_data_df['CBA04201_JQ'] - self.factor_data_df['CBA03801_JQ']) * self.factor_data_df['CBA03801'] \
                                         + self.factor_data_df['CBA03801_JQ'] / (self.factor_data_df['CBA03801_JQ'] - self.factor_data_df['CBA04201_JQ']) * self.factor_data_df['CBA04201']
        self.factor_data_df['F_currency'] = self.factor_data_df['H11025'] - self.deposit
        self.factor_data_df['F_convertible'] = self.factor_data_df['000832'] - self.deposit
        self.factor_data_df['F_equity'] = self.factor_data_df['000906'] - self.deposit
        self.factor_data_df = self.factor_data_df[self.all_factor_list]
        self.factor_data_df = (self.factor_data_df.fillna(0.0) + 1).cumprod()

        preprocessed_data = dict()
        if not (self.fund_nav_series.empty or self.factor_data_df.empty or self.fund_nav_series.reindex(date_list).interpolate().dropna().empty):
            fund_nav_series = self.fund_nav_series.reindex(date_list).interpolate().dropna().sort_index()
            fund_nav_series = fund_nav_series / fund_nav_series.iloc[0]
            fund_ret_series = fund_nav_series.pct_change().dropna()
            factor_data_df = self.factor_data_df / self.factor_data_df.iloc[0]
            factor_data_df = factor_data_df.reindex(date_list).fillna(method='ffill')
            factor_data_df = factor_data_df.pct_change().dropna().reindex(fund_ret_series.index).fillna(0.)
            assert (fund_ret_series.shape[0] == factor_data_df.shape[0])

            for date in compute_list:
                interval_data = dict()
                if self.fund_type == 'mutual':
                    if self.frequency != 'season':
                        start = self.calendar_df[self.calendar_df['CALENDAR_DATE'] == date][self.backup_period].iloc[0] if len(self.calendar_df[self.calendar_df['CALENDAR_DATE'] == date].dropna(subset=[self.backup_period])) >= 1 else self.start_date
                        end = date
                    else:
                        start = self.calendar_df[(self.calendar_df['CALENDAR_DATE'] <= date) & (self.calendar_df['IS_QUARTER_END'] == 1)]['CALENDAR_DATE'].iloc[-2] if len(self.calendar_df[(self.calendar_df['CALENDAR_DATE'] <= date) & (self.calendar_df['IS_QUARTER_END'] == 1)].dropna(subset=['CALENDAR_DATE'])) >= 2 else self.start_date
                        end = self.calendar_df[(self.calendar_df['CALENDAR_DATE'] <= date) & (self.calendar_df['IS_QUARTER_END'] == 1)]['CALENDAR_DATE'].iloc[-1] if len(self.calendar_df[(self.calendar_df['CALENDAR_DATE'] <= date) & (self.calendar_df['IS_QUARTER_END'] == 1)].dropna(subset=['CALENDAR_DATE'])) >= 1 else date
                else:
                    if self.frequency != 'season':
                        start = self.calendar_df[self.calendar_df['CALENDAR_DATE'] == date][self.backup_period].iloc[0] if len(self.calendar_df[self.calendar_df['CALENDAR_DATE'] == date].dropna(subset=[self.backup_period])) >= 1 else self.start_date
                        end = date
                    else:
                        start = self.calendar_df[(self.calendar_df['CALENDAR_DATE'] <= date) & (self.calendar_df['IS_QUARTER_END'] == 1)]['CALENDAR_DATE'].iloc[-3] if len(self.calendar_df[(self.calendar_df['CALENDAR_DATE'] <= date) & (self.calendar_df['IS_QUARTER_END'] == 1)].dropna(subset=['CALENDAR_DATE'])) >= 3 else self.start_date
                        end = self.calendar_df[(self.calendar_df['CALENDAR_DATE'] <= date) & (self.calendar_df['IS_QUARTER_END'] == 1)]['CALENDAR_DATE'].iloc[-1] if len(self.calendar_df[(self.calendar_df['CALENDAR_DATE'] <= date) & (self.calendar_df['IS_QUARTER_END'] == 1)].dropna(subset=['CALENDAR_DATE'])) >= 1 else date
                actual_point = len(self.fund_nav_series[(self.fund_nav_series.index >= start) & (self.fund_nav_series.index <= end)])
                all_points = len([x for x in date_list if (x >= start) and (x <= end)])
                if all_points == 0 or float(actual_point / all_points) <= 0.7:
                    interval_data['fund_ret_series'] = pd.Series()
                    interval_data['factor_data_df'] = pd.DataFrame()
                else:
                    interval_data['fund_ret_series'] = fund_ret_series[(fund_ret_series.index >= start) & (fund_ret_series.index <= end)]
                    interval_data['factor_data_df'] = factor_data_df[(factor_data_df.index >= start) & (factor_data_df.index <= end)]
                preprocessed_data[date] = interval_data
        else:
            for date in compute_list:
                interval_data = dict()
                interval_data['fund_ret_series'] = pd.Series()
                interval_data['factor_data_df'] = pd.DataFrame()
                preprocessed_data[date] = interval_data
        self.preprocessed_data = preprocessed_data

    def _campisi_analysis(self):
        if self.fund_type == 'mutual':
            bond_fund_position = HBDB().read_bond_fund_position_given_code(self.fund_id)
            bond_fund_position = bond_fund_position.rename(columns={'jjdm': 'FUND_CODE', 'jsrq': 'REPORT_DATE', 'gptzsz': 'STOCK', 'kzzsz': 'CONVERTIBLE'})
            if len(bond_fund_position) == 0:
                params_list, p_value_list, r_square_list, total_return_list, attr_return_list = [], [], [], [], []
                for date, period_data in self.preprocessed_data.items():
                    params = pd.DataFrame(index=[date], columns=['const'] + self.all_factor_list)
                    p_value = pd.DataFrame(index=[date], columns=['const'] + self.all_factor_list)
                    r_square = pd.DataFrame(index=[date], columns=['r_square'])
                    total_return = pd.DataFrame(index=[date], columns=['total_return'])
                    attr_return = pd.DataFrame(index=[date], columns=['special'] + self.all_factor_list)
                    params_list.append(params)
                    p_value_list.append(p_value)
                    r_square_list.append(r_square)
                    total_return_list.append(total_return)
                    attr_return_list.append(attr_return)
                params_df = pd.concat(params_list)
                p_value_df = pd.concat(p_value_list)
                r_square_df = pd.concat(r_square_list)
                total_return_df = pd.concat(total_return_list)
                attr_return_df = pd.concat(attr_return_list)
                return params_df, p_value_df, r_square_df, total_return_df, attr_return_df
            bond_fund_position['REPORT_DATE'] = bond_fund_position['REPORT_DATE'].astype(str)
            bond_fund_position = bond_fund_position.sort_values('REPORT_DATE')
            bond_fund_position = bond_fund_position.merge(self.calendar_df[['CALENDAR_DATE', 'TRADE_DATE']], left_on='REPORT_DATE', right_on='CALENDAR_DATE', how='left')
        else:
            bond_fund_position = pd.DataFrame()

        params_list, p_value_list, r_square_list, total_return_list, attr_return_list = [], [], [], [], []
        for date, period_data in self.preprocessed_data.items():
            if self.fund_type == 'mutual':
                bond_fund_position_date = bond_fund_position[bond_fund_position['TRADE_DATE'] <= date].iloc[-2:]
                bond_fund_position_stock = True if bond_fund_position_date['STOCK'].fillna(0.0).sum() > 0 else False
                bond_fund_position_convertible = True if bond_fund_position_date['CONVERTIBLE'].fillna(0.0).sum() > 0 else False
                factor_list = self.all_factor_list[:5]
                if bond_fund_position_convertible:
                    factor_list = factor_list + ['F_convertible']
                if bond_fund_position_stock:
                    factor_list = factor_list + ['F_equity']
            else:
                factor_list = self.all_factor_list[:5]
                if self.convertible:
                    factor_list = factor_list + ['F_convertible']
                if self.equity:
                    factor_list = factor_list + ['F_equity']
            
            if period_data['fund_ret_series'].empty or period_data['factor_data_df'].empty:
                logger.warning("{}: fund_ret_series or factor_data_df is empty !".format(date))
                params = pd.DataFrame(index=[date], columns=['const'] + self.all_factor_list)
                p_value = pd.DataFrame(index=[date], columns=['const'] + self.all_factor_list)
                r_square = pd.DataFrame(index=[date], columns=['r_square'])
                total_return = pd.DataFrame(index=[date], columns=['total_return'])
                attr_return = pd.DataFrame(index=[date], columns=['special'] + self.all_factor_list)
            else:
                # 标准化回归后得暴露
                fund_ret_series = period_data['fund_ret_series']
                fund_ret_series = (fund_ret_series - fund_ret_series.mean()) / fund_ret_series.std(ddof=1)
                factor_data_df = period_data['factor_data_df'][factor_list]
                factor_data_df = (factor_data_df - factor_data_df.mean()) / factor_data_df.std(ddof=1)
                factor_data_df = sm.add_constant(factor_data_df)
                if factor_data_df.dropna(axis=1).shape[1] < len(factor_list) + 1 or factor_data_df.dropna(axis=0).shape[0] < fund_ret_series.shape[0]:
                    logger.warning("{}: factor_data_df lack data !".format(date))
                    params = pd.DataFrame(index=[date], columns=['const'] + self.all_factor_list)
                    p_value = pd.DataFrame(index=[date], columns=['const'] + self.all_factor_list)
                    r_square = pd.DataFrame(index=[date], columns=['r_square'])
                    total_return = pd.DataFrame(index=[date], columns=['total_return'])
                    attr_return = pd.DataFrame(index=[date], columns=['special'] + self.all_factor_list)
                else:
                    if self.fund_type == 'private' and self.frequency == 'season':
                        n = len(fund_ret_series)
                        half_life = int(n / 2)
                        weight = np.array([2 ** (-i / half_life) for i in range(n)])
                        weight = (weight / weight.sum())[::-1]
                        result = sm.WLS(fund_ret_series, factor_data_df.values, weights=weight).fit()
                    else:
                        result = sm.OLS(fund_ret_series, factor_data_df.values).fit()
                    params = pd.DataFrame(result.params, columns=[date]).T
                    params.columns = ['const'] + factor_list
                    p_value = pd.DataFrame(result.pvalues, columns=[date]).T
                    p_value.columns = ['const'] + factor_list
                    r_square = pd.DataFrame(index=[date], columns=['r_square'], data=[result.rsquared])

                    # 非标准化回归后得暴露求收益
                    fund_ret_series_forret = period_data['fund_ret_series']
                    factor_data_df_forret = period_data['factor_data_df'][factor_list]
                    factor_data_df_forret = sm.add_constant(factor_data_df_forret)
                    if self.fund_type == 'private' and self.frequency == 'season':
                        n = len(fund_ret_series)
                        half_life = int(n / 2)
                        weight = np.array([2 ** (-i / half_life) for i in range(n)])
                        weight = (weight / weight.sum())[::-1]
                        result_forret = sm.WLS(fund_ret_series_forret, factor_data_df_forret.values, weights=weight).fit()
                    else:
                        result_forret = sm.OLS(fund_ret_series_forret, factor_data_df_forret.values).fit()
                    params_forret = pd.DataFrame(result_forret.params, columns=[date]).T
                    params_forret.columns = ['const'] + factor_list

                    fund_nav_series = period_data['fund_ret_series']
                    fund_nav_series.iloc[0] = 0.0
                    fund_nav_series = (fund_nav_series + 1).cumprod()
                    total_return_data = fund_nav_series.iloc[-1] / fund_nav_series.iloc[0] - 1.0
                    total_return = pd.DataFrame(index=[date], columns=['total_return'], data=[total_return_data])
                    factor_nav_df = period_data['factor_data_df'][factor_list]
                    factor_nav_df.iloc[0] = 0.0
                    factor_nav_df = (factor_nav_df + 1).cumprod()
                    factor_ret_df = pd.DataFrame(factor_nav_df.iloc[-1] / factor_nav_df.iloc[0] - 1.0, columns=[date]).T
                    attr_return = params_forret[factor_list] * factor_ret_df[factor_list]
                    attr_return.loc[date, 'special'] = total_return_data - attr_return.sum(axis=1).loc[date]
                    attr_return = attr_return[['special'] + factor_list]

                    params = params.T.reindex(['const'] + self.all_factor_list).T
                    p_value = p_value.T.reindex(['const'] + self.all_factor_list).T
                    attr_return = attr_return.T.reindex(['special'] + self.all_factor_list).T
            params_list.append(params)
            p_value_list.append(p_value)
            r_square_list.append(r_square)
            total_return_list.append(total_return)
            attr_return_list.append(attr_return)
            print('[{0}]/[{1}]归因done'.format(self.fund_id, date))
        params_df = pd.concat(params_list)
        p_value_df = pd.concat(p_value_list)
        r_square_df = pd.concat(r_square_list)
        total_return_df = pd.concat(total_return_list)
        attr_return_df = pd.concat(attr_return_list)
        return params_df, p_value_df, r_square_df, total_return_df, attr_return_df

    def get_all(self):
        params_df, p_value_df, r_square_df, total_return_df, attr_return_df = self._campisi_analysis()

        params_df.columns = [col + '_exp' for col in list(params_df.columns)]
        p_value_df.columns = [col + '_p' for col in list(p_value_df.columns)]
        attr_return_df.columns = [col + '_attr_ret' for col in list(attr_return_df.columns)]
        result = pd.concat([params_df, p_value_df, r_square_df, total_return_df, attr_return_df], axis=1)
        result = result.reset_index().rename(columns={'index': 'attr_date'})
        result_columns = list(result.columns)
        result['fund_id'] = self.fund_id
        result['backup_period'] = self.backup_period
        result = result[['fund_id', 'backup_period'] + result_columns]
        return result


if __name__ == '__main__':
    # 分析样本：成立满6个月的短期纯债型、中长期纯债型、混合债券型一级、混合债券型二级、被动指数型债券、增强指数型债券、可转换债券型、平衡混合型、灵活配置型、偏债混合型基金
    bond_fund = HBDB().read_bond_fund_info()
    bond_fund = bond_fund.rename(columns={'jjdm': 'FUND_CODE', 'jjmc': 'FUND_FULL_NAME', 'jjjc': 'FUND_SHORT_NAME', 'clrq': 'BEGIN_DATE', 'zzrq': 'END_DATE', 'ejfl': 'FUND_TYPE_2ND', 'kffb': 'OPEN_CLOSE'})
    bond_fund['FUND_TYPE_2ND'] = bond_fund['FUND_TYPE_2ND'].replace({'21': '被动指数型债券', '22': '短期纯债型', '23': '混合债券型一级', '24': '混合债券型二级', '25': '增强指数型债券', '27': '中长期纯债型',  '28': '可转换债券型', '34': '平衡混合型', '35': '灵活配置型', '38': '偏债混合型'})
    bond_fund['OPEN_CLOSE'] = bond_fund['OPEN_CLOSE'].replace({'0': '开放式', '1': '封闭式'})
    bond_fund = bond_fund.dropna(subset=['BEGIN_DATE'])
    bond_fund['END_DATE'] = bond_fund['END_DATE'].fillna(20990101)
    bond_fund['BEGIN_DATE'] = bond_fund['BEGIN_DATE'].astype(int)
    bond_fund['END_DATE'] = bond_fund['END_DATE'].astype(int)
    bond_fund['BEGIN_DATE'] = bond_fund['BEGIN_DATE'].astype(str)
    bond_fund['END_DATE'] = bond_fund['END_DATE'].astype(str)
    date_before = (datetime.today() - timedelta(180)).strftime('%Y%m%d')
    bond_fund = bond_fund[bond_fund['BEGIN_DATE'] <= date_before]

    # 公募样例：
    calendar_df = get_calendar_df()
    factor_data_df = get_factor_data_df()
    res = BondCampisiAttribution(calendar_df, factor_data_df, fund_id='110008', fund_type='mutual', start_date='20171201', end_date='20211231', frequency='day', backup_period='近三月').get_all()
    res = BondCampisiAttribution(calendar_df, factor_data_df, fund_id='110008', fund_type='mutual', start_date='20171201', end_date='20211231', frequency='season').get_all()
    print(res)

    # 分析样本：成立满6个月的债券型私募基金
    private_bond_fund = HBDB().read_private_bond_fund_info()
    private_bond_fund = private_bond_fund.rename(columns={'jjdm': 'FUND_CODE', 'jjmc': 'FUND_FULL_NAME', 'jjjc': 'FUND_SHORT_NAME', 'clrq': 'BEGIN_DATE', 'zzrq': 'END_DATE', 'jjfl': 'FUND_TYPE_1ST'})
    private_bond_fund['FUND_TYPE_1ST'] = private_bond_fund['FUND_TYPE_1ST'].replace({'2': '债券型'})
    private_bond_fund = private_bond_fund.dropna(subset=['BEGIN_DATE'])
    private_bond_fund['END_DATE'] = private_bond_fund['END_DATE'].fillna(20990101)
    private_bond_fund['BEGIN_DATE'] = private_bond_fund['BEGIN_DATE'].astype(int)
    private_bond_fund['END_DATE'] = private_bond_fund['END_DATE'].astype(int)
    private_bond_fund['BEGIN_DATE'] = private_bond_fund['BEGIN_DATE'].astype(str)
    private_bond_fund['END_DATE'] = private_bond_fund['END_DATE'].astype(str)
    date_before = (datetime.today() - timedelta(180)).strftime('%Y%m%d')
    private_bond_fund = private_bond_fund[private_bond_fund['BEGIN_DATE'] <= date_before]

    # 私募样例：
    calendar_df = get_calendar_df()
    factor_data_df = get_factor_data_df()
    res = BondCampisiAttribution(calendar_df, factor_data_df, fund_id='P11701', fund_type='private', start_date='20171201', end_date='20211231', frequency='day', backup_period='近六月', convertible=True, equity=True).get_all()
    res = BondCampisiAttribution(calendar_df, factor_data_df, fund_id='P11701', fund_type='private', start_date='20171201', end_date='20211231', frequency='season').get_all()
    print(res)

    # # 跑批
    # import time
    # t1 = time.time()
    # res1_list, res2_list = [], []
    # for idx, fund_id in enumerate(bond_fund['FUND_CODE'].unique().tolist()[480:]):
    #     res1 = BondCampisiAttribution(fund_id=fund_id, start_date='20220616', end_date='20220616', frequency='day', backup_period='近三月').get_all()
    #     res2 = BondCampisiAttribution(fund_id=fund_id, start_date='20211231', end_date='20211231', frequency='season').get_all()
    #     res1_list.append(res1)
    #     res2_list.append(res2)
    #     print(idx, len(bond_fund))
    # res1 = pd.concat(res1_list)
    # res2 = pd.concatappend(res2_list)
    # res1.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/campisi/res1.hdf', key='table', mode='w')
    # res2.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/campisi/res2.hdf', key='table', mode='w')
    # t2 = time.time()
    #
    # t3 = time.time()
    # res1_list, res2_list = [], []
    # for idx, fund_id in enumerate(bond_fund['FUND_CODE'].unique().tolist()):
    #     res1 = BondCampisiAttribution(fund_id=fund_id, start_date=bond_fund[bond_fund['FUND_CODE'] == fund_id]['BEGIN_DATE'].values[0], end_date='20220616', frequency='day', backup_period='近三月').get_all()
    #     res2 = BondCampisiAttribution(fund_id=fund_id, start_date=bond_fund[bond_fund['FUND_CODE'] == fund_id]['BEGIN_DATE'].values[0], end_date='20211231', frequency='season').get_all()
    #     res1_list.append(res1)
    #     res2_list.append(res2)
    #     print(idx, len(bond_fund))
    # res1 = pd.concat(res1_list)
    # res2 = pd.concatappend(res2_list)
    # res1.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/campisi/res1_bach.hdf', key='table', mode='w')
    # res1.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/campisi/res2_bach.hdf', key='table', mode='w')
    # t4 = time.time()
    #
    # print(t2 - t1, t4 - t3)


    # # 样图
    # import matplotlib.pyplot as plt
    # plt.rcParams['font.sans-serif'] = ['SimHei']
    # plt.rcParams['axes.unicode_minus'] = False
    # line_color_list = ['#F04950', '#6268A2', '#959595', '#333335', '#EE703F', '#7E4A9B', '#8A662C', '#44488E']
    # bar_color_list = ['#C94649', '#EEB2B4', '#E1777A', '#D57C56', '#E39A79', '#DB8A66', '#E5B88C',
    #                   '#8588B7', '#B4B6D1', '#55598D', '#628497', '#A9C6CB', '#866EA9', '#B79BC7',
    #                   '#7D7D7E', '#CACACA', '#A7A7A8', '#606063', '#C4C4C4', '#99999B', '#B7B7B7']
    # periods = 4
    #
    # params_df = res['params_df'].drop('CONST', axis=1)
    # params_df = params_df.sort_index().iloc[-periods:].fillna(0.0)
    # xlabels = list(params_df.columns)
    # xlabels = [xlabel.replace('F_level', '系统').replace('F_slope', '斜率').replace('F_credit', '信用风险溢价').replace('F_default', '违约风险补偿').replace('F_currency', '货币').replace('F_convertible', '转债').replace('F_equity', '权益')for xlabel in xlabels]
    # theta = np.linspace(0.0, 2.0 * np.pi, len(xlabels), endpoint=False)
    # ydata = params_df.values
    # labels = list(params_df.index)
    # labels = [label[:6].replace('03', 'Q1').replace('06', 'Q2').replace('09', 'Q3').replace('12', 'Q4') for label in labels]
    # for i in range(periods):
    #     plt.polar(np.concatenate((theta, [theta[0]])), np.concatenate((ydata[i], [ydata[i][0]])), label=labels[i], color=line_color_list[i])
    # plt.gca().set_xticks(theta)
    # plt.gca().set_xticklabels(xlabels)
    # plt.legend(ncol=periods, bbox_to_anchor=(1, -0.05))
    # plt.title('各因子暴露')
    # plt.tight_layout()
    # plt.show()
    #
    # p_value_df = res['p_value_df'].drop('CONST', axis=1)
    # p_value_df = p_value_df.sort_index().iloc[-periods:].fillna(0.0)
    # xlabels = list(p_value_df.columns)
    # xlabels = [xlabel.replace('F_level', '系统').replace('F_slope', '斜率').replace('F_credit', '信用风险溢价').replace('F_default', '违约风险补偿').replace('F_currency', '货币').replace('F_convertible', '转债').replace('F_equity', '权益') for xlabel in xlabels]
    # theta = np.linspace(0.0, 2.0 * np.pi, len(xlabels), endpoint=False)
    # ydata = p_value_df.values
    # labels = list(p_value_df.index)
    # labels = [label[:6].replace('03', 'Q1').replace('06', 'Q2').replace('09', 'Q3').replace('12', 'Q4') for label in labels]
    # for i in range(periods):
    #     plt.polar(np.concatenate((theta, [theta[0]])), np.concatenate((ydata[i], [ydata[i][0]])), label=labels[i], color=line_color_list[i])
    # plt.gca().set_xticks(theta)
    # plt.gca().set_xticklabels(xlabels)
    # plt.legend(ncol=periods, bbox_to_anchor=(1, -0.05))
    # plt.title('各因子暴露p值')
    # plt.tight_layout()
    # plt.show()
    #
    # r_square_series = res['r_square_series']
    # r_square_series = r_square_series.sort_index().iloc[-periods:].fillna(0.0)
    # r_square_series.index = map(lambda x: x[:6].replace('03', 'Q1').replace('06', 'Q2').replace('09', 'Q3').replace('12', 'Q4'), r_square_series.index)
    # plt.bar(range(periods), r_square_series, width=0.5, tick_label=list(r_square_series.index), label='回归日期', color=bar_color_list[0])
    # plt.legend()
    # plt.title('回归R2')
    # plt.tight_layout()
    # plt.show()
    #
    # params_df = res['params_df'].drop('CONST', axis=1)
    # params_df = params_df.sort_index().fillna(0.0)
    # params_df.index = map(lambda x: datetime.strptime(str(x), '%Y%m%d'), params_df.index)
    # labels = list(params_df.columns)
    # labels = [label.replace('F_level', '系统').replace('F_slope', '斜率').replace('F_credit', '信用风险溢价').replace('F_default', '违约风险补偿').replace('F_currency', '货币').replace('F_convertible', '转债').replace('F_equity', '权益') for label in labels]
    # params_df.columns = labels
    # fig, ax = plt.subplots(figsize=(10, 5))
    # for i in range(len(labels)):
    #     plt.plot(params_df.index, params_df[labels[i]].values, label=labels[i], color=line_color_list[i])
    # plt.legend(ncol=len(labels), bbox_to_anchor=(0.9, -0.20))
    # plt.xticks(rotation=90)
    # plt.title('各因子暴露变化')
    # plt.tight_layout()
    # plt.show()